pro canopus,axes=ax,maria=mar,mpa=mpa,bars=bar,asi=asi,$
            gsc=gsc,usgs=usgs,inter=inter,jpn=jpn,labels=lab,oval=oval,$
            con_color=c_col,grid_color=g_col,sym_color=s_col,$
            bar_color=b_col,ov_color=o_col,logo_col=l_col,$
            ut=ut,edfl=edfl,sphere=sphere,numlats=numlt,numlongs=numlg,$
            thick=thk,fill=fll,south=sth,north=nth,west=wst,east=est,$
            sphr_lat=sphrlat,sphr_long=sphrlon,$
            charsize=csz,symsize=symsz,linestyle=lstl,noclip=noclip,$
            nocont=nocont,nologo=nologo,nogrid=nogrid,nofill=nofill,$
            twoletter=let2
;
;+
; NAME:
;     CANOPUS
; PURPOSE:
;     To draw a map of CANOPUS and/or related sites.
; CATEGORY:
;     Plotting, CANOPUS
; CALLING SEQUENCE:
;     CANOPUS
;
;     CANOPUS [,/MARIA] [,/MPA] [,/ASI] [,/BARS] [,/GSC] [,/USGS] [,/INTER] 
;             [,/JPN] [,/OVAL] [,UT=time] 
;             [,/LABELS] [,CHARSIZE=size]
;             [,LINESTYLE=style] 
;             [,SYMSIZE=size] [,/FILL] [,THICK=thickness] 
;             [,CON_COLOR=color] [,GRID_COLOR=color] [,SYM_COLOR=color]
;             [,BAR_COLOR=color] [,OV_COLOR=color] [,LOGO_COLOR=color]
;             [,/EDLF] 
;             [,/NOCLIP]
;             [,S_LAT=south] [,N_LAT=north] [,E_LONG=east] [,W_LONG=west]
;             [,/SPHERE] [,SPHR_LAT=lat] [,SPHR_LONG=long]
;             [,NUMLATS=number] [,NUMLONGS=number]
;             [,/NOCONT] [,/NOLOGO] [,/NOGRID] [,/NOFILL]
;             [,/TWOLETTER]
;
; INPUTS:
;     none
; OPTIONAL INPUT PARAMETERS: none
; KEYWORD PARAMETERS:
;
;    MARIA  -  indicates MARIA sites are to be plotted. Plot symbol = 6
;              The user may define a plot symbol by setting
;              MARIA = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    MPA    -  indicates MPA sites are to be plotted. Plot symbol = 2
;              The user may define a plot symbol by setting
;              MPA = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
;                                                                    
;    ASI    -  indicates ASI sites are to be plotted. Plot symbol = 4
;              The user may define a plot symbol by setting
;              ASI = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    BARS   -  indicates BARS sites are to be plotted. Plot symbol = 5
;              The user may define a plot symbol by setting
;              BARS = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    GSC    -  indicates Geological Survey of Canada magnetometer sites 
;              are to be plotted.  Plot symbol = 7
;              The user may define a plot symbol by setting
;              GSC = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    USGS   -  indicates U.S. Geological Survey magnetometer sites are 
;              to be plotted. Plot symbol = 1
;              The user may define a plot symbol by setting
;              USGS = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    INTER  -  indicates Major International magnetometer sites are 
;              to be plotted. Plot symbol = 1
;              The user may define a plot symbol by setting
;              INTER = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
;
;    JPN  -    indicates magnetometer for the Japanese EXOS-D sites are 
;              to be plotted. Plot symbol = 1
;              The user may define a plot symbol by setting
;              JPN = sym_array  where sym_array is a (2,N) array of
;                                 vertices, with elements (0,i) containing
;                                 the X coordinate and (1,i) containing
;                                 the Y coordinate (see USERSYM in the
;                                 IDL manual)
; 
;    OVAL - indicates that an Auroral Oval is to be drawn in the map (sphere)
;           /OVAL causes the outline of the auroral oval to be drawn with
;                 solid lines (6:40 UT Dec 16, 1972 if keyword UT not used)
;           OVAL=int_number  where int_number > 0 
;                                  oval is drawn using linestyle = int_number
;                            where int_number = -1
;                                  oval is drawn using a 'grey' fill pattern
;
;    UT - Determines the Universal Time to be used to postition the auroral
;         oval. Keyword OVAL must be set. 
;
;    LABELS  - indicates plotted sites are to be labeled with a 4 letter
;              site code. Default is no labels
; 
;    CHARSIZE - determines the character size to be used for labels.
;               Default = 1.0
; 
;    LINESTYLE - determines the line style to be used to draw the continent
;                outlines Default=0
;
;    SYMSIZE - determines the symbol size to be used in plotting.
;              Default = 1.0
; 
;    FILL - indicates that the user made symbols are to be filled in
;
;    THICK - determines the line style to be used to draw the symbols
;
;    CON_COLOR -color of the continent lines Default=!p.color
;
;    GRID_COLOR -color of the grid lines Default=!p.color
;
;    SYM_COLOR -color of the symbols Default=!p.color
;
;    BAR_COLOR -color of the BARS filled area Default=200 (if color device)
;
;    OV_COLOR -color of the OVAL fill Default=100 (if color device) 
;
;    LOGO_COLOR -color of the CSA logo Default=!p.color
;
;    EDFL  -  determines if data will be plotted in EDFL or geodetic
;             coordinates. Default is geodetic
; 
;    NOCLIP - determines if data point will be clipped (excluded and not
;             plotted) if they are outside the plot box. Default is clipping.
;             This has no effect if /SPHERE is used.
; 
;    SOUTH  - determines the southern latitude limit of the map. 
;             Default = 40.  geo (50. edfl) 
;             This has no effect if /SPHERE is used.
; 
;    NORTH  - determines the northern latitude limit of the map.
;             Default = 85.  geo (90. edfl) 
;             This has no effect if /SPHERE is used.
;
;    EAST   - determines the eastern longitude limit of the map
;             Default = 310.  geo (20. edfl) 
;             This has no effect if /SPHERE is used.
; 
;    WEST   - determines the westerm longitude limit of the map.
;             Default = 220.  geo (290. edfl) 
;             This has no effect if /SPHERE is used.
; 
;    SPHERE - determines if the map will be plotted on a sphere or a 
;             flat projection. Default is a flat projection
; 
;    SPHR_LAT - determines the latitude of the sphere perspective.
;               Default = 30. geo (20. edfl) 
; 
;    SPHR_LONG - determines the longtitude of the sphere perspective.
;                Default = 260. geo (330. edfl) 
;
;    NUMLATS   - Used to set the number of latitude grid points the sphere
;                 Default = 37 (1 point = 5 degrees)
;
;    NUMLONGS  - Used to set the number of longitude grid points the sphere
;                 Default = 37 (1 point = 10 degrees)
;
;    NOCONT - indicates that the drawing of the continent outlines is to
;             be suppressed
; 
;    NOLOGO - indicates the the drawing of the CSA logo and 'CANOPUS' title
;             is to be suppressed
;
;    NOGRID - indicates the the drawing of the Grid lines are to be
;             suppressed
; 
; OUTPUTS:
;    none
; OPTIONAL OUTPUT PARAMETERS:
;    none
; COMMON BLOCKS:
;    none
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;    Note; when plotting on a sphere the user must set the aspect ratio 
;          (using SET_VIEWPORT) in the calling routine. 
;          A X:Y ratio of 0.64:1.0 produces a circle using TEK 4014. 
;          Ex. SET_VIEWPORT,0.1,0.74,0.0,1.0 
;
; MODIFICATION HISTORY:
;   Created - 08 Feb 1991 - J.D. Trickey
;   additional options added - 10 Apr 1991 - J.D. Trickey
;-
;=======================================================================
;
if keyword_set(csz) then lab_size=csz else lab_size=1.0
if keyword_set(symsz) then sym_size=symsz else sym_size=1.0
if keyword_set(ax) then axes_val=1 else axes_val=0
if keyword_set(noclip) then noclip_val=1 else noclip_val=0
if keyword_set(nogrid) then grid_val=0 else grid_val=1
if keyword_set(fll) then fill_val=1 else fill_val=0
if keyword_set(thk) then thick_val=thk else thick_val=0
if keyword_set(c_col) then con_color=c_col else con_color=!p.color
if keyword_set(g_col) then grid_color=g_col else grid_color=!p.color
if keyword_set(s_col) then sym_color=s_col else sym_color=!p.color
if keyword_set(b_col) then bars_color=b_col else bars_color=200
if keyword_set(o_col) then oval_color=o_col else oval_color=100
if keyword_set(l_col) then logo_color=l_col else logo_color=!p.color
if n_elements(numlt) ne 0 then nlat_val=numlt else nlat_val=37 
if n_elements(numlg) ne 0 then nlon_val=numlg else nlon_val=37 
if keyword_set(nocont) then begin
     nocont_val=1 
     cont_val=0
endif else begin
     nocont_val=0
     cont_val=1
endelse
;
; MAKE LABELS IF DESIRED
if keyword_set(lab) then begin
   if keyword_set(let2) then let2_val=1 else let2_val=0
   canopus_labels,mar_labels,mpa_labels,asi_labels,bar_labels,$
                   gsc_labels,usg_labels,int_labels,jpn_labels,twoletter=let2_val
   mar_lat_pos=[-0.5,-0.5,0.5,0.0,1.0,-0.5,-1.0,0.5,-0.5,1.0,$
                -1.0,-1.0,-0.5,0.0]
   mar_lon_pos=[1.0, 1.0, 1.0,1.0,0.0, 1.0, 0.5,0.5, 1.3,0.0,$
                2.0 , 1.2, 1.0, 0.0]
   mpa_lat_pos=[0.5,1.0,-0.5,-1.0]
   mpa_lon_pos=[0.5,0.0,1.0,0.5]
   asi_lat_pos=[-0.5]
   asi_lon_pos=[1.0]
endif
;
; SET TO EDFL COORDINATES
if keyword_set (edfl) then begin
   edfl_val=1
endif else begin
   edfl_val=0
endelse
;   if n_elements(sth) ne 0 then south=sth else south=50.
;   if n_elements(nth) ne 0 then north=nth else north=90.
;   if n_elements(wst) ne 0 then west=wst else west=-70.
;   if n_elements(est) ne 0 then east=est else east=20.
;   if n_elements(sphrlat) ne 0 then lat_val=sphrlat else lat_val=20.
;   if n_elements(sphrlon) ne 0 then long_val=sphrlon else long_val=330.
;
;   mar_lats=[63.883,65.229,66.268,68.621,70.374,$
;             61.385,57.732,66.556,72.394,65.333,$
;             67.323,67.396,63.233]
;   mar_longs=[336.205,336.671,336.682,336.682,338.923,$
;              336.419,335.079,313.205,311.295,324.380,$
;              277.477,300.580,315.304]
;;
;   mpa_lats=[66.556,70.374,63.883,57.732]
;   mpa_longs=[313.205,338.923,336.205,335.079]
;;
;   asi_lats=[63.883]
;   asi_longs=[336.205]
;   angle=findgen(46)*8.0
;   rad=fltarr(46)+1.0
;   asi_cir_lats=rad*sin(angle/!radeg)
;   asi_cir_longs=rad*cos(angle/!radeg)
;;
;   bar_lats=[60.579,58.506]
;   bar_longs=[325.269,338.116]
;   bar_line1_lats=[68.063,58.506,65.537]
;   bar_line1_longs=[342.252,338.116,331.997]
;   bar_line2_lats=[63.940,60.579,68.921]
;   bar_line2_longs=[340.047,325.269,339.352]
;   bar_area_lat=[63.035,65.537,68.921,68.891,68.063,63.940,63.035]
;   bar_area_long=[334.559,331.997,339.352,340.342,342.252,340.047,334.559]
;;
;   oval_high_lats=[65.8140,65.9034,65.7307,65.0281,67.7332,72.5911,$
;                  73.4207,74.4878,73.9196,75.1242,75.3024,76.3193,$
;                  76.1944,76.0515,75.2963,74.6667,75.2172,73.9925,$
;                  71.5769,71.3747,68.7043,67.0052,65.8140]
;   oval_low_lats=[61.8084,60.9754,60.9843,61.4073,62.6420,64.9595,$
;                 66.9725,68.6209,70.5725,73.3362,73.9445,73.8277,$
;                 74.0458,73.6862,73.3315,72.0220,71.2030,70.2127,$
;                 67.7527,66.3912,64.8196,63.0433,61.8084]
;   oval_high_longs=[340.233,351.442,2.18013,13.1019,24.7953,37.3525,$
;                   51.9083,68.2705,86.6225,103.309,119.580,161.020,$
;                   175.164,189.492,205.690,223.343,243.422,263.570,$
;                   282.313,300.200,314.928,328.155,340.233]
;   oval_low_longs=[339.820,351.260,2.19677,13.2332,24.9755,38.2696,$
;                  54.0277,70.5616,88.0964,104.415,120.121,161.880,$
;                  175.412,189.590,205.424,222.819,242.131,262.061,$
;                  280.911,298.443,313.809,327.505,339.820]
;   oval_high_mlt=((12.+(oval_high_longs-80.)/15.)+24.) MOD 24.
;   oval_low_mlt=((12.+(oval_low_longs-80.)/15.)+24.) MOD 24.
;   gsc_lats=[86.212,71.781,66.308,76.247,82.354,$
;             57.127,54.220,53.318,62.810,60.994,$
;             68.741,53.471,82.021]
;   gsc_longs=[91.223,333.299,336.652,318.040,331.053,$
;              333.833,24.742,358.816,357.234,313.307,$
;              308.971,303.276,285.788]               
;;
;   usg_lats=[62.704,70.420,66.769,68.910,71.789, $
;             60.813,64.829,54.266,46.588] 
;   usg_longs=[267.698,266.300,266.872,268.294,250.642, $
;              286.248,265.978,310.505,356.583]
;;
;   int_lats=[52.583,69.087,64.578,64.659,74.743, $
;             49.590,72.785,41.307,63.402,63.476, $
;             65.928,54.619,59.235,75.072,63.177, $
;             66.205,62.345,63.515,65.455,84.058, $
;             53.054]
;   int_longs=[76.489,169.713,243.967,153.771, 35.173, $
;              89.275,142.951,172.509,112.513,106.297, $
;              65.674,110.718, 82.730,119.646,117.132, $
;              38.077,111.037,189.344,106.602, 30.642, $
;             193.421]
;;
; SET TO GEODETIC COORDINATES
   if n_elements(sth) ne 0 then south=sth else south=40.
   if n_elements(nth) ne 0 then north=nth else north=85.
   if n_elements(wst) ne 0 then west=wst else west=220.
   if n_elements(est) ne 0 then east=est else east=310.
   if n_elements(sphrlat) ne 0 then lat_val=sphrlat else lat_val=60.
   if n_elements(sphrlon) ne 0 then long_val=sphrlon else long_val=260.
;
   mar_lats=[56.376,57.717,58.763,61.106,62.824,   $
             53.856,50.199,60.017,65.754,58.222,   $
             64.048,61.756,56.657,69.53]
   mar_longs=[265.36,265.83,265.92,265.95,267.89,  $
              265.34,263.96,248.05,248.75,256.32,  $
              220.89,238.77,248.79,266.47]
;
   mpa_lats=[60.017,62.824,56.376,50.199]
   mpa_longs=[248.05,267.89,265.36,263.96]
;
   asi_lats=[56.376]
   asi_longs=[265.36]
   angle=findgen(91)*4.0
   rad=fltarr(91)+1.0
   asi_cir_lats=rad*sin(angle/!radeg)*3.1+56.376
   asi_cir_longs=rad*cos(angle/!radeg)*5.6+265.36
;
   bar_lats=[53.47,50.9]
   bar_longs=[256.23,266.54]
   bar_line1_lats=[60.38,50.9,58.01]
   bar_line1_longs=[270.29,266.54,262.19]
   bar_line2_lats=[56.22,53.47,61.27]
   bar_line2_longs=[268.41,256.23,268.14]
   bar_area_lat=[55.41,58.01,61.27,61.23,60.38,56.22,55.41]
   bar_area_long=[263.96,262.19,268.14,268.88,270.29,268.41,263.96]
;
   oval_low_lats=[54.03,53.11,53.31,54.19,56.21,$
                  59.74,63.48,67.21,71.51,76.32,$
                  79.03,82.29,82.08,80.59,78.47,$
                  75.11,71.79,68.31,63.86,60.83,$
                  58.13,55.64,54.03]
   oval_low_longs=[268.12,277.52,286.65,295.87,305.44,$ 
                   315.66,327.70,340.53,354.15,  5.53,$ 
                    21.85, 91.77,118.17,141.13,161.58,$ 
                   178.09,195.00,210.60,223.61,236.51,$ 
                   247.89,258.32,268.12]
   oval_high_lats=[58.11,58.19,58.23,57.95,61.44,$
                   67.29,69.50,72.37,74.29,77.67,$
                   80.10,84.68,84.07,82.61,80.02,$
                   77.16,74.90,71.39,67.20,65.42,$
                   61.87,59.59,58.11]
   oval_high_longs=[268.65,277.44,285.97,294.91,303.39,$
                    310.21,320.36,331.50,347.54,  0.29,$
                     17.43, 89.79,122.36,147.86,167.01,$
                    183.88,202.44,215.94,227.35,240.23,$
                    249.88,259.45,268.65]                  
   e_o_high_lats=[65.8140,65.9034,65.7307,65.0281,67.7332,72.5911,$
                  73.4207,74.4878,73.9196,75.1242,75.3024,76.3193,$
                  76.1944,76.0515,75.2963,74.6667,75.2172,73.9925,$
                  71.5769,71.3747,68.7043,67.0052,65.8140]
   e_o_low_lats=[61.8084,60.9754,60.9843,61.4073,62.6420,64.9595,$
                 66.9725,68.6209,70.5725,73.3362,73.9445,73.8277,$
                 74.0458,73.6862,73.3315,72.0220,71.2030,70.2127,$
                 67.7527,66.3912,64.8196,63.0433,61.8084]
   e_o_high_longs=[340.233,351.442,2.18013,13.1019,24.7953,37.3525,$
                   51.9083,68.2705,86.6225,103.309,119.580,161.020,$
                   175.164,189.492,205.690,223.343,243.422,263.570,$
                   282.313,300.200,314.928,328.155,340.233]
   e_o_low_longs=[339.820,351.260,2.19677,13.2332,24.9755,38.2696,$
                  54.0277,70.5616,88.0964,104.415,120.121,161.880,$
                  175.412,189.590,205.424,222.819,242.131,262.061,$
                  280.911,298.443,313.809,327.505,339.820]
   oval_high_mlt=((12.+(e_o_high_longs-80.)/15.)+24.) MOD 24.
   oval_low_mlt=((12.+(e_o_low_longs-80.)/15.)+24.) MOD 24.
;
   gsc_lats=[82.50,64.30,58.80,69.12,74.69,$
             49.63,47.57,45.42,55.27,54.62,$
             62.40,48.33,76.20]
   gsc_longs=[297.60,264.00,265.90,254.98,265.00,$
              262.87,307.28,284.30,282.22,246.67,$
              245.60,236.63,242.60]
;
   usg_lats=[61.200,68.130,64.880,66.570,71.300, $
             57.100,63.300,48.300,38.200]
   usg_longs=[210.100,214.430,211.950,214.720,203.250, $
              224.700,209.900,242.900,282.600]
;
   int_lats=[52.42,77.72,66.17,73.50,69.30, $
             51.53,80.60,52.27,68.60,67.80, $ 
             64.18,59.95,60.13,80.05,69.00, $
             61.20,67.40,71.60,69.70,77.50, $
             62.00]
   int_longs=[355.93,104.28,190.17,80.40,306.50, $
                9.97, 58.00,104.30, 27.48, 20.40,$
              338.30, 30.70,358.82, 18.25, 33.05,$
              314.60, 26.60,128.90, 19.00,290.80,$
              129.60]
;
   jpn_lats=[80.00,74.70, 69.10, 63.80, 64.90, 54.80,$
             58.90, 52.00, 53.80, 60.70, 53.40,$
             53.60, 52.70, 56.20, 50.20, 49.20,$
             52.20, 49.10, 48.30, 45.40, 78.90,$
             71.30, 61.20, 69.30, 40.10]
   jpn_longs=[ 274.10, 265.10, 255.00, 291.50, 234.50, 293.10,$
               237.20, 264.50, 258.80, 224.90, 299.60,$
               254.10, 258.70, 239.10, 268.50, 275.10,$
               252.80, 268.80, 307.30, 284.30,  11.90,$
               203.20, 314.60,  20.80, 254.80]
;
if n_elements(lstl) ne 0 then begin
   linestyle=!p.linestyle
   !p.linestyle=lstl
endif
;
; DRAW MAP ON SPHERE
if keyword_set(sphere) then begin
   map_set,lat_val,long_val,/ortho,/nobord
   latdel=360./float(nlat_val-1)
   londel=360./float(nlon_val-1)
   if (not keyword_set(nologo)) then begin
      csalogo,0.9,0.82,fact=0.5,color=logo_color
      xyouts,0.9,0.91,'CANOPUS',/norm,align=0.5,size=1.5,color=logo_color
   end
;
;
; DRAW MAP (FLAT)
endif else begin 
   limit=[south,west,north,east]                                                        
   map_set,90,-100.,/lamb,limit=limit
   latdel=5.
   londel=5.
   if (not keyword_set(nologo)) then begin
      csalogo,0.8,0.8,fact=0.5,color=logo_color
      xyouts,0.84,0.76,'CANOPUS',/norm,align=0.5,size=1.5,color=logo_color
   end
endelse
;
   if keyword_set(oval) then begin         ; OVAL
      if oval gt 1 then begin
          line=oval
      endif else line=0
      if n_elements(ut) ne 0 then begin
         sun=(180.-ut*15.) MOD 360.
         e_o_low_longs=sun+(oval_low_mlt-12.)*15.  
         e_o_high_longs=sun+(oval_high_mlt-12.)*15.
         neg=where(e_o_low_longs lt 0.0)
         if neg(0) ne -1 then e_o_low_longs(neg)=e_o_low_longs(neg)+360.  
         neg=where(e_o_high_longs lt 0.0)
         if neg(0) ne -1 then e_o_high_longs(neg)=e_o_high_longs(neg)+360.  
         if keyword_set(edfl) then begin
            oval_low_longs= e_o_low_longs  
            oval_high_longs= e_o_high_longs  
         endif else begin
            gcrkm=fltarr(n_elements(e_o_low_longs))+6371.2
            erkm=fltarr(n_elements(e_o_low_longs))+6371.2
            canecdip,1,oval_low_lats,oval_low_longs,gcrkm,$
                       e_o_low_lats/!radeg,e_o_low_longs/!radeg,erkm
            canecdip,1,oval_low_lats,oval_low_longs,gcrkm,$
                       e_o_low_lats/!radeg,e_o_low_longs/!radeg,erkm
            gcrkm=fltarr(n_elements(e_o_high_longs))+6371.2
            erkm=fltarr(n_elements(e_o_high_longs))+6371.2
            canecdip,1,oval_high_lats,oval_high_longs,gcrkm,$
                       e_o_high_lats/!radeg,e_o_high_longs/!radeg,erkm
            canecdip,1,oval_high_lats,oval_high_longs,gcrkm,$
                       e_o_high_lats/!radeg,e_o_high_longs/!radeg,erkm
            oval_low_longs= oval_low_longs*!radeg  
            oval_high_longs= oval_high_longs*!radeg  
            oval_low_lats= oval_low_lats*!radeg  
            oval_high_lats= oval_high_lats*!radeg  
         endelse
      endif
      if oval lt 0 then begin
         if keyword_set(sphere) then begin
            new_order=sort(oval_low_longs)
            oval_low_longs=smooth(oval_low_longs(new_order),3)
            oval_low_lats=smooth(oval_low_lats(new_order),3)
            new_order=sort(oval_high_longs)
            oval_high_longs=smooth(oval_high_longs(new_order),3)
            oval_high_lats=smooth(oval_high_lats(new_order),3)
            latx=[oval_low_lats,oval_low_lats(0),oval_high_lats(0),reverse(oval_high_lats)]
            longx=[oval_low_longs,oval_low_longs(0),oval_high_longs(0),reverse(oval_high_longs)]
         endif else begin
            new_order=sort(oval_low_longs)
            oval_low_longs=smooth(oval_low_longs(new_order),3)
            oval_low_lats=smooth(oval_low_lats(new_order),3)
            on_map=where(oval_low_longs gt west-30. and oval_low_longs lt east+30.)
            latx=oval_low_lats(on_map)
            longx=oval_low_longs(on_map)
            new_order=sort(oval_high_longs)
            oval_high_longs=smooth(oval_high_longs(new_order),3)
            oval_high_lats=smooth(oval_high_lats(new_order),3)
            on_map=where(oval_high_longs gt west-30. and oval_high_longs lt east+30.)
            latx=[latx,reverse(oval_high_lats(on_map)),latx(0)]
            longx=[longx,reverse(oval_high_longs(on_map)),longx(0)]-360.
         endelse
         if !d.n_colors eq 2 then begin
            polyfill,longx,latx,/data,/line_fill,$
               spacing=0.7,orient=45.,$ 
               noclip=noclip_val                                   
         endif else begin
            polyfill,longx,latx,/data,color=oval_color,$
               noclip=noclip_val                                   
         endelse
      endif else begin
         if keyword_set(sphere) then begin
            new_order=sort(oval_low_longs)
            oval_low_longs=smooth(oval_low_longs(new_order),3)
            oval_low_lats=smooth(oval_low_lats(new_order),3)
            new_order=sort(oval_high_longs)
            oval_high_longs=smooth(oval_high_longs(new_order),3)
            oval_high_lats=smooth(oval_high_lats(new_order),3)
            latx=[oval_low_lats,oval_low_lats(0)]
            longx=[oval_low_longs,oval_low_longs(0)]
            oplot_drop,longx,latx,linestyle=line,$ 
                       noclip=noclip_val                                   
            latx=[oval_high_lats,oval_high_lats(0)]
            longx=[oval_high_longs,oval_high_longs(0)]
            oplot_drop,longx,latx,linestyle=line,$ 
                       noclip=noclip_val                                   
         endif else begin
            on_map=where(oval_low_longs gt west-30. and oval_low_longs lt east+30.)
            latx=fltarr(n_elements(oval_low_lats))+99999.9
            longx=fltarr(n_elements(oval_low_longs))+99999.9
            temp=smooth(oval_low_lats,3)
            latx(on_map)=temp(on_map)
            temp=smooth(oval_low_longs,3)
            longx(on_map)=temp(on_map)-360.
            oplot_drop,longx,latx,linestyle=line,$ 
                       noclip=noclip_val                                   
            on_map=where(oval_high_longs gt west-30. and oval_high_longs lt east+30.)
            latx=fltarr(n_elements(oval_high_lats))+99999.9
            longx=fltarr(n_elements(oval_high_longs))+99999.9
            temp=smooth(oval_high_lats,3)
            latx(on_map)=temp(on_map)
            temp=smooth(oval_high_longs,3)
            longx(on_map)=temp(on_map)-360.
            oplot_drop,longx,latx,linestyle=line,$ 
                       noclip=noclip_val                                   
         endelse
      endelse
   endif
   if n_elements(lstl) ne 0 then begin
      !p.linestyle=linestyle
   endif
   if keyword_set(bar) then begin        ; BARS
      if n_elements(bar) gt 1 then begin
         usersym,bar,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=5
      oplot,bar_longs-360.,bar_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color
      oplot,bar_line1_longs-360.,bar_line1_lats,linestyle=2,$ 
            noclip=noclip_val,color=bars_color                         
      oplot,bar_line2_longs-360.,bar_line2_lats,linestyle=2,$ 
            noclip=noclip_val,color=bars_color                      
      if (not keyword_set(nofill)) then begin
         if !d.n_colors eq 2 then begin
             polyfill,bar_area_long-360.,bar_area_lat,/data,/line_fill,$
             spacing=0.2,orient=30.,$ 
               noclip=noclip_val                                   
         endif else begin
             polyfill,bar_area_long-360.,bar_area_lat,/data,color=bars_color,$ 
               noclip=noclip_val                                   
         endelse
      endif
   endif
   !p.linestyle=0
   if cont_val eq 1 then map_continents,color=con_color
   if n_elements(lstl) ne 0 then begin
      !p.linestyle=linestyle
   endif
;
if keyword_set(sphere) then begin
   if grid_val eq 1 then begin
      if edfl_val eq 0 then begin
         map_grid,latdel=latdel,londel=londel,color=grid_color
      endif else begin
         gclat=fltarr(180)
         gclon=fltarr(180)
         temp_x=fltarr(180)
         temp_long=(findgen(180)*2)/!radeg
         temp_alt=fltarr(180)+6370.12
         eflag=1
         for iii=0,8 do begin
            temp_lat=(fltarr(180)+iii*10.)/!radeg
            gcrkm=temp_alt
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            gcrkm=temp_alt-(gcrkm-temp_alt)
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            oplot,gclon*!radeg,gclat*!radeg,linestyle=1
         endfor
;
         gclat=fltarr(90)
         gclon=fltarr(90)
         temp_x=fltarr(90)
         temp_lat=(findgen(90)*2-90.)/!radeg
         temp_alt=fltarr(90)+6370.12
         eflag=1
         for iii=0,35 do begin
            temp_long=(fltarr(90)+iii*10.)/!radeg
            gcrkm=temp_alt
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            gcrkm=temp_alt-(gcrkm-temp_alt)
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            oplot,gclon*!radeg,gclat*!radeg,linestyle=1
         endfor
      endelse
   endif
;
endif else begin 
   if grid_val eq 1 then begin
      if edfl_val eq 0 then begin
         map_grid,latdel=latdel,londel=londel,color=grid_color
         xyouts,east+0.5,49.7,'50',orient=east-270.,size=lab_size
         xyouts,east+0.5,59.7,'60',orient=east-270.,size=lab_size
         xyouts,east+0.5,69.7,'70',orient=east-270.,size=lab_size
         xyouts,east+0.5,79.7,'80',orient=east-270.,size=lab_size
         xyouts,-60.,south-2.,'300',orient=40,size=lab_size,align=0.5
         xyouts,-70.,south-2.,'290',orient=30,size=lab_size,align=0.5
         xyouts,-80.,south-2.,'280',orient=20,size=lab_size,align=0.5
         xyouts,-120.,south-2.,'240',orient=-20,size=lab_size,align=0.5
         xyouts,-130.,south-2.,'230',orient=-30,size=lab_size,align=0.5
      endif else begin
         gclat=fltarr(60)
         gclon=fltarr(60)
         temp_x=fltarr(60)
         temp_long=(findgen(60)*2-90.)/!radeg
         temp_alt=fltarr(60)+6370.12
         eflag=1
         for iii=0,3 do begin
            temp_lat=(fltarr(60)+50.+iii*10.)/!radeg
            gcrkm=temp_alt
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            gcrkm=temp_alt-(gcrkm-temp_alt)
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            oplot,gclon*!radeg,gclat*!radeg,linestyle=1
         endfor
;
         gclat=fltarr(40)
         gclon=fltarr(40)
         temp_x=fltarr(40)
         temp_lat=(findgen(40)+50.)/!radeg
         temp_alt=fltarr(40)+6370.12
         eflag=1
         for iii=0,12 do begin
            temp_long=(fltarr(40)-90.+iii*10.)/!radeg
            gcrkm=temp_alt
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            gcrkm=temp_alt-(gcrkm-temp_alt)
            CANECDIP,eFlag,GCLAT,GCLON,gcrkm,temp_lat,temp_long,temp_x
            oplot,gclon*!radeg,gclat*!radeg,linestyle=1
         endfor
      endelse
   endif
endelse
;
   if keyword_set(mar) then begin        ; MARIA
      if n_elements(mar) gt 1 then begin
         usersym,mar,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=6 
      oplot,mar_longs-360.,mar_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color                  
   endif
   if keyword_set(mpa) then begin        ; MPA
      if n_elements(mpa) gt 1 then begin
         usersym,mpa,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=2   
      oplot,mpa_longs-360.,mpa_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color                   
   endif
   if keyword_set(asi) then begin        ; ASI
      if n_elements(asi) gt 1 then begin
         usersym,asi,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=4
      oplot,asi_longs-360.,asi_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color 
      oplot,asi_cir_longs-360.,asi_cir_lats,linestyle=0,   $
             noclip=noclip_val,thick=thick_val,color=sym_color 
;      usersym,asi_cir_longs,asi_cir_lats
;      oplot,asi_longs-360.,asi_lats,psym=8,symsize=8.0,$
;            linestyle=0,noclip=noclip_val,color=sym_color 
;      oplot,asi_longs-360.,asi_lats,psym=8,symsize=16.0,$
;            linestyle=0,noclip=noclip_val,color=sym_color 
   endif
   if keyword_set(gsc) then begin         ; GSC
      if n_elements(gsc) gt 1 then begin
         usersym,gsc,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=7
      oplot,gsc_longs-360.,gsc_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color               
   endif
   if keyword_set(usgs) then begin         ; USGS
      if n_elements(usgs) gt 1 then begin
         usersym,usgs,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=1
      oplot,usg_longs-360.,usg_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color                
   endif
   if keyword_set(inter) then begin         ; INTER
      if n_elements(inter) gt 1 then begin
         usersym,inter,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=1
      over=fltarr(n_elements(int_longs))
      over(where(int_longs gt 180))=360.  
      oplot,int_longs-over,int_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color                   
   endif
   if keyword_set(JPN) then begin         ; JPN
      if n_elements(jpn) gt 1 then begin
         usersym,jpn,thick=thick_val,fill=fill_val
         symbol=8
      endif else symbol=1
      over=fltarr(n_elements(jpn_longs))
      over(where(jpn_longs gt 180))=360.  
      oplot,jpn_longs-over,jpn_lats,symsize=sym_size,psym=symbol,$ 
            noclip=noclip_val,thick=thick_val,color=sym_color                   
   endif
;
   if keyword_set(lab) then begin
      if keyword_set(mar) then begin
         number=n_elements(mar_labels)
         for i=0,number-1 do xyouts,mar_longs(i)-360.+mar_lon_pos(i),$
             mar_lats(i)+mar_lat_pos(i),mar_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color   
      endif
      if keyword_set(mpa) then begin
         number=n_elements(mpa_labels)
         for i=0,number-1 do xyouts,mpa_longs(i)-360.+mpa_lon_pos(i),$
             mpa_lats(i)+mpa_lat_pos(i),mpa_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(asi) then begin
         number=n_elements(asi_labels)
         for i=0,number-1 do xyouts,asi_longs(i)-360.+asi_lon_pos(i),$
             asi_lats(i)+asi_lat_pos(i),asi_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(bar) then begin
         number=n_elements(bar_labels)
         for i=0,number-1 do xyouts,bar_longs(i)-360.+1.,$
             bar_lats(i)-0.5,bar_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(gsc) then begin
         number=n_elements(gsc_labels)
         for i=0,number-1 do xyouts,gsc_longs(i)-360.+1.,$
             gsc_lats(i)-0.5,gsc_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(usgs) then begin
         number=n_elements(usg_labels)
         for i=0,number-1 do xyouts,usg_longs(i)-360.+1.,$
             usg_lats(i)-0.5,usg_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(inter) then begin
         number=n_elements(int_labels)
         over=fltarr(n_elements(int_longs))
         over(where(int_longs gt 180))=360.  
         for i=0,number-1 do xyouts,int_longs(i)-over(i)+1.,$
             int_lats(i)-0.5,int_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
      if keyword_set(jpn) then begin
         number=n_elements(jpn_labels)
         over=fltarr(n_elements(jpn_longs))
         over(where(jpn_longs gt 180))=360.  
         for i=0,number-1 do xyouts,jpn_longs(i)-over(i)+1.,$
             jpn_lats(i)-0.5,jpn_labels(i),size=lab_size,$ 
            noclip=noclip_val,color=sym_color                   
      endif
   endif
;
return
end
