PRO plot_sphere,lat_val,long_val,continent=cont,edfl=dipole,horizon=hor,$
    axes=ax,nlat=nlt,nlong=nlg,limit=lmt,scolor=sclr
;+
; NAME:
;    PLOT_SPHERE
; PURPOSE:
;    To plot a sphere or globe. The point of view of the sphere is specified
;    by LAT and LONG. A map of the Earth may be drawn on the sphere in
;    either goegraphic or eccentric dipole (EDFL) coordinates.
; CATEGORY:
;    plotting
; CALLING SEQUENCE:
;    PLOT_SPHERE,LAT,LONG   [,/CONTINENT [,/EDFL]]
;                           [,/HORIZON]  [,/AXES]
;                           [,NLAT=value] [,NLONG=value]
;                           [,LIMIT=value]
;                           [,SCOLOR=value]
; INPUTS:
;    LAT  real value  - latitude coresponding to the center of the
;                       view of the sphere
;    LONG real value  - longitude coresponding to the center of the
;                       view of the sphere
; OPTIONAL INPUT PARAMETERS: 
;    None
; KEYWORD PARAMETERS:
;    CONTINENT  - If set, the continents and major islands of the world
;                 will be drawn on the sphere, corresponding the the 
;                 given LAT and LONG
;    EDFL       - If set, the continents will be drawn in EDFL coordinates,
;                 and LAT and LONG will be assumed to be in EDFL. 
;                 CONTINENT must also be specified.
;    HORIZON    - If set, a horizon or outline will be drawn around the
;                 sphere.
;    AXES       - If set, the axes generation and rounding will no long
;                 be suppressed. This suppression is done by setting !X.STYLE
;                 and !Y.STYLE to 5, temporarily. If AXES is set, these system
;                 variables are left untouched, so can be set by the main 
;                 program.
;    NLAT       - Used to set the number of latitude grid points the sphere
;                 Default = 37 (1 point = 5 degrees)
;    NLONG      - Used to set the number of longitude grid points the sphere
;                 Default = 37 (1 point = 10 degrees)
;    LIMIT      - Used to set the maximum and minimum of the plot axes.
;                 The sphere is plotted between -1 and 1 for both X and
;                 Y and so the axes are set to these maximums, but LIMIT 
;                 may be used to set the limits to a larger value. The
;                 sphere will still have the same limits, so the user may
;                 use this to plot things above the sphere's surface.
;    SCOLOR     - determines the color of the sphere (lat-long grid and
;                 horizon). Default=0. The color of the continent outlines
;                 is determined by !P.COLOR, which has no effect on the
;                 sphere color.
; OUTPUTS:
;    none
; OPTIONAL OUTPUT PARAMETERS:
;    None
; COMMON BLOCKS:
;    PLOT_SPHR,LATV,LONGV
;      LATV  real value  - latitude coresponding to the center of the
;                          view of the sphere
;      LONGV real value  - longitude coresponding to the center of the
;                          view of the sphere
; SIDE EFFECTS:
;    The values of !X.STYLE and !Y.STYLE are temporarily set to 5 during
;    plotting. This may be prevented using the /AXES keyword.
;    !P.Psym and !P.Linestyle are temporarily changed while the sphere
;    is plotted, but are returned to the previous values before the continents
;    are plotted. 
; RESTRICTIONS: None
; PROCEDURE:
;    Note; the user must set the aspect ratio (using SET_VIEWPORT) in the
;          calling routine. A X:Y ratio of 0.64:1.0 produces a circle using
;          TEK 4014. Ex. SET_VIEWPORT,0.1,0.74,0.0,1.0 
; MODIFICATION HISTORY:
;    Created in IDL 2 - Jan. 1991 - J.D. Trickey
;-
;	
common plot_sphr,latv,lonv
;
latv=lat_val
lonv=long_val
;	
symbol=!p.psym
linetype=!P.linestyle  
color=!p.color
;
thet=(90-lat_val)/!radeg 
phet=long_val/!radeg
rot1=[[cos(phet),sin(phet),0],[-sin(phet),cos(phet),0],[0,0,1.0]]
rot2=[[cos(thet),0,-sin(thet)],[0,1.0,0],[sin(thet),0,cos(thet)]]
;
if n_elements(nlt) ne 0 then begin
     n_lats=nlt
endif else begin
     n_lats=37 
endelse
if n_elements(nlg) ne 0 then begin
     n_lons=nlg
endif else begin
     n_lons=37 
endelse
;
; get map if desired
   if keyword_set(cont) then begin
   	GET_LUN,LUN
      if keyword_set(dipole) then begin
   	   OPENR,LUN,'IDL_dir:[maps]CONTINENT_EDFL.DAT',/variable
      endif else begin
    	   OPENR,LUN,'IDL_dir:[maps]CONTINENT.DAT/UNFORMATTED'
      endelse
   endif
;
; draw sphere
;
IF (not keyword_set(ax)) then begin
   xstyle=!x.style
   ystyle=!y.style
   !x.style=5
   !y.style=5
endif
if keyword_set(lmt) then limit=lmt else limit=1.
!p.psym=3
plot,[-1.,-1.,1.,1.]*limit,[1.,-1.,-1.,1.]*limit
;
if n_elements(sclr) ne 0 then begin
   !p.color=sclr
endif else !p.color=0
;
NPTS=0   ; a 2 byte integer
MINMAX=FLTARR(4)
;define the coordinate grid
if(n_lats gt 0 and n_lons gt 0) then begin
    the=(180.0/(n_lats-1))*findgen(n_lats)/!radeg
    phi=(360.0/(n_lons-1))*findgen(n_lons)/!radeg
;    !p.psym=3
    !p.linestyle=1
    for i=0,n_lats-1 do begin 
      th=the(i)              
      for j=0,n_lons-1 do begin 
         ph=phi(j)               
         pos=[sin(th)*cos(ph),sin(th)*sin(ph),cos(th)]        
         p_n=pos#rot1#rot2                                
         if (p_n(2) ge 0.0) then oplot,[p_n(1),p_n(1)],[-p_n(0),-p_n(0)] 
      endfor 
    endfor
endif
!p.psym=0
!p.LINEstyle=0
if keyword_set(hor) then begin
    x=findgen(361)
    y=sin(x/!radeg)
    x=cos(x/!radeg)
    oplot,x,y
endif

!p.psym=symbol
!p.LINEstyle=linetype
!p.color=color
if keyword_set(cont) then begin
   WHILE NOT EOF(LUN) DO BEGIN
    readu,LUN,NPTS,MINMAX & NPTS=NPTS/2
    LON=FLTARR(NPTS) & LAT=FLTARR(NPTS) & readu,LUN,LAT,LON
    nx=fltarr(npts) & ny=nx & nz=nx 
    p=where(lon gt 180.0) & if(p(0) ne -1) then lon(p)=lon(p)-360.0 
    p=where(lon lt -180.) & if(p(0) ne -1) then lon(p)=lon(p)+360.0
    for i=0,npts-1 do begin & the=lat(i)  & phi=lon(i)             
      if ((abs(the) le 90.)and (abs(phi) le 360.)) then begin 
          th=(90-the)/!radeg & ph=lon(i)/!radeg            
          pos=[sin(th)*cos(ph),sin(th)*sin(ph),cos(th)]    
          pn=pos#rot1#rot2                                 
          nx(i)=-pn(0) & ny(i)=pn(1) & nz(i)=pn(2)         
      endif else begin                                     
          nx(i)=-10 & ny(i)=-10 & nz(i)=-1 
      endelse                                              
    endfor
    T=where(nz ge 0.0)
    if (t(0) ge 0) then oPLOT,Ny(T),nx(T)	   
   ENDWHILE
  	FREE_LUN,LUN			    ;corner if device is a 4014 term.
endif
IF (not keyword_set(ax)) then begin
   !x.style=xstyle
   !y.style=ystyle
endif
RETURN
END
