;+
; NAME:
;	VELO_VECT  (originally velovect)
; PURPOSE:
;	Produce a two dimensional velocity field plot.
;	Draw a directed arrow at each point showing the direction and magnitude 
;	of the field.
; CATEGORY:
;	Plotting
; CALLING SEQUENCE:
;	VELO_VECT,U,V  ...  or:
;	VELO_VECT,U,V,XX,YY
; INPUTS:
;	U = The X component of the two dimensional field.  Both
;		U and V must be two dimensional and have the same
;		dimensions.
;	V = The Y component of the two dimensional field.  The
;		vector at point (i,j) has the magnitude of
;		(U(i,j)^2 + V(i,j)^2)^0.5 and a direction of
;		ATAN2(V(i,j),U(i,j)).
; OPTIONAL INPUT PARAMETERS:
;	XX = Optional abcissae values, 1 or 2 dimensions.
;	     First dimension of XX must equal first dimension of U and V.
;	YY = Optional ordinate values, 1 or 2 dimensions.
;	     Last dimension of YY must equal second dimension of U and V.
;	If Polar is present and non-zero then XX and YY represent R and
;	Theta (radians)
; KEYWORD INPUT PARAMETERS:
;	POLAR: If present and non-zero, XX and YY are interpreted as R and
;		Theta (radians).
;	AXES:  If present and non-zero, rectangular axes are plotted.
;		Default is to use established data scaling and to plot no axes.
;	ARROW: If present and non-zero, vectors are plotted as a line with an 
;		arrow head and missing points are marked with periods.
;		Else, vectors are plotted as a line pointing from tiny filled
;		circle at the tail and missing points are marked with the same 
;		circles.
;	CALLEN = plotted length of a hypothetical vector of magnitude CALMAG,
;		in units of X or R. Default is X (or R) length of a cell.
;		If zero supplied, default will be used and returned in CALLEN.
;	CALMAG = magnitude, in units of U and V, of a hypothetical vector to
;		be plotted with length CALLEN. Default is the maximum magnitude
;		of (U,V).
;		If zero supplied, default will be used and returned in CALMAG.
;	MISSING = missing data value.  (U,V) Vectors with a LENGTH longer
;		than missing are ignored.
;	DOTS: If present and non-zero, place a dot at each missing point. 
;		Otherwise draw nothing for missing points.  Has effect only if
;		Missing is specified.
;	DOT_SIZE = size of dots marking vector tails and/or missing points.
;		If value is 0.0 then a period (of size 1.0) is used instead 
;		of a filled circle.
;	XRANGE = X-axis range. 2-element FLOAT vector.
;	YRANGE = Y-axis range. 2-element FLOAT vector.
; SIDE EFFECTS:
;	Plotting on the selected device is performed.  System
;	variables concerning Plotting are changed.
; MODIFICATION HISTORY:
;	Dms, Rsi, Oct, 1983.
;	For Sun, DMS, RSI, April, 1989.
;	Ian Grant, HIA/NRCC, 11 June 1990. Added Aspect, Polar, Axes, Arrow.
;	IFG, HIA/NRCC, 1991: Converted to IDL2. Added Plot_keywords.
;	IFG, 16 May 1991: Added NGOOD GT 0 checks.
;	IFG,  6 Aug 1991: Changed dot from filled to outline 16-sided polygon.
;	IFG, 11 Aug 1992: Fixed bug in YY 1D to 2D conversion, with LEADDIM1.
;	IFG, 12 Aug 1992: Fixed more bugs in YY 1D to 2D conversion.
;	D.F. Tanguay, 25 Jan. 1993:  renamed module to VELO_VECT from VELOVECT 
;	    to elliminate confusion with USERLIB routine.
;	    - Also elliminated use of aspectd_xony and leaddim1 function calls.
;-

PRO VELO_VECT,U,V,XX,YY, $
   Polar = polar, Axes = axes, Arrow = arrow, $
   Callen = callen, Calmag = calmag, $
   Missing = Missing, Dots = dots, Dot_size = dot_size, $
   xrange=xrange, yrange=yrange

	on_error,2                      ;Return to caller if an error occurs
	s = size(u)
	t = size(v)
	if s(0) ne 2 then begin 
baduv:	 message, 'U and V parameters must be 2D and same size.'
		endif
	if total(abs(s(0:2)-t(0:2))) ne 0 then goto,baduv
;
	if n_params(0) ge 3 then $
           x = xx $
        else $
           x = findgen(s(1))
        sx = size(x)
        case sx(0) of
           1: begin
              if sx(1) ne s(1) then goto, badxy
              x_1d = x
              x = fltarr(s(1),s(2))
              for i=0,s(2)-1 do $
                 x(0,i) = x_1d
              end
           2: begin
              if (sx(1) ne s(1)) or (sx(2) ne s(2)) then goto, badxy
              end
           else: goto, badxy
           endcase
;
	if n_params(0) ge 4 then $
           y = yy $
        else $
           if keyword_set(polar) then $
              y = 2.0*!pi/float(s(2)) * findgen(s(2)) $
           else $
              y = findgen(s(2))
        sy = size(y)
        case sy(0) of
           1: begin
              if sy(1) ne s(2) then goto, badxy
	      s_y = size(y)
              y_1d = reform(y,[1,s_y(1:s_y(0))])
              y = fltarr(s(1),s(2))
              for i=0,s(1)-1 do $
                 y(i,0) = y_1d
              end
           2: begin
              if (sy(1) ne s(1)) or (sy(2) ne s(2)) then goto, badxy
              end
           else: goto, badxy
           endcase

            goto, skip_badxy
badxy:      message, 'X and Y arrays have incorrect size.'
skip_badxy:

;
	if n_elements(missing) le 0 then missing = 1.0e30
	if n_elements(aspect) le 0 then aspect = 1.0
	if n_elements(dot_size) le 0 then dot_size = 0.18
	if not keyword_set(axes) then axes = !false

	mag = sqrt(u^2+v^2)		;magnitude.
		;Subscripts of good elements
	nbad = 0			;# of missing points
	if n_elements(missing) gt 0 then begin
		good = where(mag lt missing, ngood)
		if keyword_set(dots) then bad = where(mag ge missing, nbad)
	endif else begin
		ngood = n_elements(mag)
		good = lindgen(ngood)
	endelse

	if ngood gt 0 then begin
		mag = mag(good)			;Discard missing values
		maxmag = max(mag)
		ugood = u(good)
		vgood = v(good)
	endif

	;get scaling

	xmax = max(x, min=xmin)
	xd = float(xmax-xmin)/(s(1)-1)

	if axes then begin
		; For each of X and Y, choose axis range from, in decreasing 
		; priority: XRANGE, !X.RANGE, range of values in X.
		if (not keyword_set(xrange)) then begin
			if !x.range(0) ne !x.range(1) then $
				xrange=!x.range $
			else $
				if keyword_set(polar) then begin
					xrange = [-xmax-xd, xmax+xd]
				endif else begin
					xrange = [xmin-xd, xmax+xd]
				endelse
		endif
		if (not keyword_set(yrange)) then begin
			if !y.range(0) ne !y.range(1) then $
				yrange=!y.range $
			else $
				if keyword_set(polar) then begin
					yrange = xrange
				endif else begin
					ymax = max(y, min=ymin)
					yd = float(ymax-ymin)/(s(2)-1)
					yrange = [ymin-yd, ymax+yd]
				endelse
		endif
	endif

	if ngood gt 0 then begin
		if not keyword_set(callen) then $
			callen = xd
		if (not keyword_set(calmag)) then $
			calmag = maxmag
	endif

if keyword_set(axes) then $
	; Make grid and set axis system variables.
	plot, fltarr(1), fltarr(1), /nodata, $
		xrange=xrange, yrange=yrange
;
;	Plot good points
;	----------------

	if ngood gt 0 then begin
		aspect_xony = float(!d.x_size)/float(!d.y_size)
		; Calculate scaled vectors in terms of data units
		vecx = float(callen)/calmag*ugood   ; distance in X data units
		vecy = float(callen)/calmag*vgood $ ; distance in X data units
			* !x.s(1) $		; convert to X norm'd units
			* aspect_xony $ 	; convert to Y norm'd units
			/ !y.s(1)		; convert to Y data units
;
		r = .3				;len of arrow head
		angle = 22.5 * !dtor		;Angle of arrowhead
		st = r * sin(angle)		;sin 22.5 degs * length of head
		ct = r * cos(angle)
;
		if keyword_set(arrow) or (dot_size eq 0.0) then begin
			; Period
			dot_sym = 3
			dot_sym_size = 1.0
		endif else begin
			; Tiny filled circle
			dot_sym = 8
			dot_sym_size = dot_size
			symarg = (2.0*!pi/16.0) * findgen(17)
			usersym, cos(symarg), sin(symarg)
		endelse
;
		for i=0,ngood-1 do begin	;Each good point
			ix = good(i) mod s(1)    ; Find 2D indices of point
			iy = good(i) / s(1)
			x0 = x(ix,iy)    ;get coords of start
			y0 = y(ix,iy)
			if keyword_set(polar) then begin
				; Convert (x0,y0) from polar to cartesian
				r = x0
				t = y0
				x0 = r*cos(t)
				y0 = r*sin(t)
				endif
			dx = vecx(i)    ;get coords of end
			x1 = x0 + dx
			dy = vecy(i)
			y1 = y0 + dy
			if keyword_set(arrow) then $
				plots, noclip=!p.noclip, $
				[x0,x1,x1-(ct*dx-st*dy),x1,x1-(ct*dx+st*dy)], $
				[y0,y1,y1-(ct*dy+st*dx),y1,y1-(ct*dy-st*dx)] $
			else begin
				plots, noclip=!p.noclip,[x0],[y0], $
					psym = dot_sym, symsize = dot_sym_size
				plots, noclip=!p.noclip,[x0,x1],[y0,y1]
				endelse
			endfor
		endif ; ngood gt 0
;
;	Plot missing points
;	-------------------

	if nbad gt 0 then begin		;Dots for missing?
		if keyword_set(polar) then begin
			for i=0,nbad-1 do begin
				ix = bad(i) mod s(1)
				iy = bad(i) / s(1)
				r = x(ix,iy)
				t = y(ix,iy)
				; Convert (x0,y0) from polar to cartesian
				oplot, [r*cos(t)], [r*sin(t)], $
					psym = dot_sym, symsize = dot_sym_size
				endfor
		endif else $
			ix = bad mod s(1)    ; Find 2D indices of point
			iy = bad / s(1)
			oplot, x(ix,iy), y(ix,iy), $
				psym = dot_sym, symsize = dot_sym_size
		endif ; nbad gt 0

return
end
