;+
; FILENAME:
;       CADI_IQ2VEL
;
;+
; NAME:
;       CADI_IQ2XAP
; PURPOSE:
;       Determine cross-amplitudes and cross-phases from CADI IQ data.
; CALLING SEQUENCE:
;       CADI_XAP = CADI_IQ2XAP( CADI_IQ )
; INPUTS:
;       CADI_IQ: Variable of CADI structure containing IQ data.
; OUTPUTS:
;       CADI_XAP = Variable of CADI structure containing XAP (cross-amplitude
;          and phase) data. Each element of CADI_XAP.DATA coresponds to one
;          element of CADI_IQ.DATA. No element of CADI_XAP.DATA is generated
;          for those elements of CADI_IQ.DATA for which cross-amplitudes and
;          phases cannot be calulated (because I=Q=0 for one or more antennas).
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 9 Sep 1994: Written by adapting CADI_MD_SOURCES.
;-

function cadi_iq2xap, cadi

on_error, 0
if cadi.datatype ne 'IQ' then $
   message, 'Argument does not have IQ (in-phase and quadrature) data'
nsources = cadi.ndata
if nsources eq 0 then $
   goto, form_nullresult
header = cadi.header
d = cadi.data
;stop
site = cadi_site(header.site, header.time)      ; For antenna order and polarity
nantennas = header.nantennas

; Reject bad antennas
; -------------------

; Determine which sources have (I,Q) NE (0,0) for all antennas.
; We will reject the other sources.
;
; (I EQ 0) AND (Q EQ 0)   is 1 for each antenna with I=Q=0 (a bad antenna),
;                            0 otherwise.
; TOTAL( ,1)    returns the number of bad antennas for each element of D.
; WHERE         returns the indices of the elements of D with no bad antennas.
;
;%% Note "I OR Q" is several times faster than "(I EQ 0) AND (Q EQ 0)" (but
;%% gives a different answer).
;%% Might be able to find a faster logic expression.

i = reform(d.iq(0,*), nantennas, nsources)
q = reform(d.iq(1,*), nantennas, nsources)
niqsources = nsources
wgood = where( round(total( (i eq 0) and (q eq 0), 1)) eq 0, nsources)

;%% IDL requires different syntaxes for DIM(SOURCES) EQ/GT 1
;%% Until I think of a tidy workaround, don't process NSOURCES EQ 1
;%% 940909 CADI_MD_XAMPPHA: Try to accept NSOURCES=1.%% if nsources le 1 then $
if nsources eq 0 then $
   goto, form_nullresult
if nsources lt niqsources then $
   d = d(wgood)

; accept only strong echoes using following procedure
;whigh = where(round(total(((i*i + q*q) lt 10),1)) eq 0 , hsources)
;if hsources eq 0 then $
;   goto, form_nullresult
;if hsources lt nsources then begin $
;   d = d(whigh)
;   nsources = hsources
;   endif

; Decode I and Q and convert to float
; -----------------------------------

w = where(d.iq ge 128, wcount)
iq = float(d.iq)                ; 3D array (2,4,nsources)
if wcount gt 0 then $
   iq(w) = iq(w) - 256.0
; Find cross-power and cross-phase
; --------------------------------

xpow = fltarr(nsources,2)
xpha = xpow

for pairx=0,1 do begin                  ; Nominally NS (EW) for PAIRX = 0 (1)
   ant0 = site.antennax(2*pairx)        ; Nominally N or E
   ant1 = site.antennax(2*pairx+1)      ; Nominally S or W
   ; For possible efficiency, could explicitly calculate RE and IM from IQ
   ; without COMPLEX, possibly using integer arithmetic. ;%%
   s = reform(      complex( iq(0,ant0,*), iq(1,ant0,*) ) * $
               conj(complex( iq(0,ant1,*), iq(1,ant1,*) )) )
   if site.polarity(ant0) * site.polarity(ant1) eq -1 then $
      s = -s
   re = float(s)
   im = imaginary(s)
   xpow(0,pairx) = re^2 + im^2
   xpha(0,pairx) = atan(im,re)  ;%% Very occasionally this is ATAN(0,0)
 endfor
for sourcenumber = 0L, nsources-1 do begin
 ;print, sourcenumber, xpha(sourcenumber,0), xpha(sourcenumber,1)
 for pairx=0,1 do begin
   ;stop
   if pairx eq 0 then begin
         xpha(sourcenumber,0) = xpha(sourcenumber,0) + site.phasecorrectionEW
         ;print, xpha(sourcenumber,0)
         if xpha(sourcenumber,0) gt  3.14159 then xpha(sourcenumber,0) = xpha(sourcenumber,0) - 6.2832
         if xpha(sourcenumber,0) lt  -3.14159 then xpha(sourcenumber,0) = xpha(sourcenumber,0) + 6.2832
         ;print, xpha(sourcenumber,0)
    endif
   if pairx eq 1 then begin
         xpha(sourcenumber,1) = xpha(sourcenumber,1) + site.phasecorrectionNS
         ;print, xpha(sourcenumber,1)
         if xpha(sourcenumber,1) gt  3.14159 then xpha(sourcenumber,1) = xpha(sourcenumber,1) - 6.2832
         if xpha(sourcenumber,1) lt  -3.14159 then xpha(sourcenumber,1) = xpha(sourcenumber,1) + 6.2832
         ;print, xpha(sourcenumber,1)
 ;  print, sourcenumber, xpha(sourcenumber,0), xpha(sourcenumber,1)
 ;        print, s(sourcenumber)
 ;        print, iq(*,*,sourcenumber)
 ; cont_or_quit = reply_char('[C]ontinue or [Q]uit','cq','q')
;   if cont_or_quit eq 'Q' then goto, exit

       endif
   endfor
   endfor

exit:
; Create output vector of sources
; -------------------------------
;help, /mem
sources = replicate( {x:d(0).x, xpower:fltarr(2), xphase:fltarr(2)}, nsources)
sources.x = d.x
sources.xpower = transpose(xpow)
sources.xphase = transpose(xpha)
;print,sources(100)
form_result:
;help, /mem
return, create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', nsources, $
   'DATATYPE', 'XAP', $
   'HEADER', cadi.header, $
   'FREBINS', cadi.frebins, $
   'DATA', sources)

form_nullresult:

sources = -1L
nsources = 0L
goto, form_result

end

;+
; NAME:
;       CADI_IQ2VEL
; PURPOSE:
;       Fit velocities to IQ sources returned by CADI_READ_DIR.
; CALLING SEQUENCE:
;       VEL = CADI_IQ2VEL( CADI_IQ )
; INPUTS:
;       CADI_IQ: Variable of CADI structure containing IQ data.
; OUTPUTS:
;       VEL = Vector of velocity structures, one element for each time
;          represented in CADI_IQ. Times are roughly regular, with 'missing'
;          velocity values filling any gaps.
; PROCEDURE:
;       Calls the following CADI routines in sequence:
;          CADI_IQ2XAP
;          CADI_XAP_VETAOA (with /NOALIASAOA)
;          CADI_XAP2KVEC
;          CADI_KVEC2VEL
;          CADI_VEL_REGTIM
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 18 Oct 1994: Written.
;-

function cadi_iq2vel, cadi_iq
;help, /mem
on_error, 0

if cadi_iq.datatype ne 'IQ' then $
   message, 'Argument does not have IQ (in-phase and quadrature) data'

xap = cadi_iq2xap(cadi_iq)
vet = cadi_xap_vetaoa(temporary(xap),/noaliasaoa)
kvec = cadi_xap2kvec(temporary(vet))
vel = cadi_kvec2vel(temporary(kvec))
;stop
return, cadi_vel_regtim(temporary(vel))

end
