;+
; NAME:
;       CADI_VEL_EDIT
; PURPOSE:
;       Edit CADI Doppler drift velocities to reject times of low quality.
; CALLING SEQUENCE:
;       Result = CADI_VEL_EDIT( VEL )
;
;       For thresholds which seem to work well in many cases, use:
;          Result = CADI_VEL_EDIT(VEL,NSOURCE=40,AZI=20,HOR=100,VER=15,/DROP)
; INPUTS:
;       VEL = Vector of CADI velocity structures.
; KEYWORD PARAMETERS:
;       NSOURCES_THRESH = Minimum number of sources for accepted point.
;       AZI_ERR_THRESH = Maximum azimuth error for accepted point.
;       HOR_ERR_THRESH = Maximum horizontal speed error for accepted point.
;       VER_ERR_THRESH = Maximum vertical velocity error for accepted point.
;       DROP_NSOURCES: If present and nonzero, set NSOURCES field to 'missing'
;          for all missing points. Before plotting the NSOURCES field, set this
;          to plot gaps in place of low source counts.
; OUTPUTS:
;       Result = Vector of CADI velocity structures which is a subset of VEL.
;          Any element of VEL which does not satisfy all of the threshold
;          conditions specified by the keywords is rejected.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, Oct 1994: Written.
;       GRANT, UWO, 8 Feb 1995: Added handling of null argument.
;-

function cadi_vel_edit, f_in, nsources_thresh=nsources_thresh, $
   azi_err_thresh=azi_err_thresh, hor_err_thresh=hor_err_thresh, $
   ver_err_thresh=ver_err_thresh, drop_nsources=drop_nsources

on_error, 2
;stop
if (size(f_in)) (0) eq 0 then return, f_in      ; Null in, null out

max_value = 9.0e8       ; 900 MHz
missing = 1e9
lon_missing = long(missing)

if not keyword_set(nsources_thresh) then nsources_thresh = -1
if not keyword_set(azi_err_thresh) then azi_err_thresh = missing
if not keyword_set(hor_err_thresh) then hor_err_thresh = missing
if not keyword_set(ver_err_thresh) then ver_err_thresh = missing

f = f_in

; Reject any unwanted times

w = where((f.hor lt max_value) and $    ; Examine only non-missing points
          ((f.nsources lt nsources_thresh) or $
           (f.azi_err gt azi_err_thresh) or $
           (f.hor_err gt hor_err_thresh) or $
           (f.ver_err gt ver_err_thresh)), wcount)

if wcount gt 0 then begin
   f(w).azi     = missing
   f(w).hor     = missing
     f(w).horew   = missing
     f(w).horns   = missing
   f(w).ver     = missing
   f(w).azi_err = missing
   f(w).hor_err = missing
   f(w).ver_err = missing
   f(w).rms_error = missing
   endif

; If /DROP_NSOURCES then set NSOURCES to LON_MISSING for all missing points,
; even if they were missing before this routine was called.

if keyword_set(drop_nsources) then begin
   w = where((f.hor gt max_value), wcount)
   if wcount gt 0 then f(w).nsources = lon_missing
   endif

return, f
end
