;+
; NAME:
;       CADI_HTSUMM
; PURPOSE:
;       Generate summary plot of one day's h't from CADI data files.
;       Designed for MD and FHT data formats.
; CALLING SEQUENCE:
;       CADI_HTSUMM, DIR=DIR, DATESTR=DATESTR, EXT=EXT
; INPUTS:
;       DIR = Scalar string containing directory in which data is located.
;          Syntax ready to prepend onto filename e.g. 'D:\CADIDATA\'.
;          Default is ''.
;       DATESTR = string holding date of data as 6 ASCII digits YYMMDD.
;          Default is to take date from header of first .ext file in directory.
;       EXT = scalar string containing extension of data files, without period.
;          Default is 'MD1'.
; KEYWORD PARAMETERS:
;       INFORM: If present and nonzero, progress messages are printed.
;       SUBPLOT_NHOURS = Integer number of hours in each panel. Default 6.
;       NSUBPLOTS = Integer number of panels. Default 24/SUBPLOT_NHOURS.
;       START_HOUR = Integer hour at start of first panel. Default 0.
;       FREQX = scalar integer index into list of sounding frequencies in file
;          header. Selects frequency to be plotted. Default is 0.
;       ERROR = integer scalar. Returned as true if an error occurred, else
;          false.
; RESTRICTIONS:
;       %% Plots all files in directory, even if outside range of timeaxis.
;       %% Single hour is too high to fit on 5:4 plot. Use PORTRAIT in that
;          case?
; SIDE EFFECTS:
;       Generates a plot file.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 3 Feb 1995.
;      JMacD, UWO, 9 Apr, 1995.
;          added pcl plot pixel mapping
;		28 Jan 2000 by JWM Y2K fix for year
;-

pro cadi_htsumm, dir=dir, datestr=datestr_in, ext=file_ext, $
   inform=inform, error=error, $
   subplot_nhours=subplot_nhours, nsubplots=nsubplots, start_hour=start_hour, $
   freqx=freqx, heightrange = heightrange,timerange = timerange, allfreq = allfreq

on_error, 0

psav = !p
xsav = !x
ysav = !y

; Set up plot now to be ready for any 'Aborted' message.

erase
if freqx ne 10 then time

;os_chars, wild_1char=wild_1char, enddir_char=enddir_char

if not keyword_set(dir) then dir = ''
if not keyword_set(file_ext) then file_ext = 'MD1'
if not keyword_set(freqx) then freqx = 0

;%% ; Find the first file with the right extension in the directory,
;; read its header, and extract some information from the header.
;
;files=findfile(dir+'*.'+file_ext, count=nfiles)
;if nfiles eq 0 then begin
;   error_message = 'No files '+dir+'*.'+file_ext
;   goto, error_return
;   endif
;;%% h = cadi_read_md(files(0), /onlyheader, error=error)
;if in(strupcase(file_ext), ['FHT', 'FFI']) then $
;   h = cadi_read_ft(files(0), /onlyheader, error=error) $
;else $
;   h = cadi_read_md(files(0), /onlyheader, error=error)
;if error then begin
;   error_message = 'Error reading first file for header information'
;   goto, error_return
;   end

; Get header information (CADI_READ_DIR get it from first file)

h = cadi_read_dir(dir=dir, ext=file_ext, /onlyheader, error=error)
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif
max_height = h.maxheight
dh = median(difference( h.heights ))         ; Typical height step; pixel height

; Determine default pixel width DT.

;%% dt = h.dtime
if strupcase(file_ext) eq 'FHT' then $
   dt = h.nfreqs * h.dtime $
else $
   dt = h.dtime

; Input date from keyword or from first file

if not keyword_set(datestr_in) then $
   candate = canmidnight(h.time) $   ; Date from time in first file
else begin
   datestr = datestr_in ; Unnecessary?
   reads,datestr,year,month,day,format='(3i2)'       ; Date from keyword
   if year lt 54 then year = year + 100
   candate=cantime(1900+year,month,day,0,0,0)
   endelse




if not keyword_set(subplot_nhours) then subplot_nhours = 6L
if not keyword_set(nsubplots) then nsubplots = 24/subplot_nhours
if not keyword_set(start_hour) then start_hour = 0L
civil_time, year, month, day, daynum, hour, minute, second, candate ; For DAYNUM

if !d.name eq 'PS' then begin
   device, bits_per_pixel=8
   back_color = 255b    ; White
   fore_color = 0b      ; Black
 endif else if !d.name eq 'PCL' then begin
      back_color = 255b ; White
      fore_color = 0b   ; Black
endif else if !d.name eq 'PCN' then begin
      back_color = 255b ; White
      fore_color = 0b   ; Black
 endif else begin
   back_color = 255b      ; White
   !p.background = 255b
   fore_color = 0b    ; Black
   !p.color = 0
   endelse
erase


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if not keyword_set(allfreq) then begin

rx = 0L         ; Index of receiver to be plotted
!p.noerase = 1

; Set POS_REGIONS(1) and POS_IN_REGIONS(1) so as to position bottoms of
; subplots to give one hour a square aspect ratio.

pos_regions = [0.1,-1,0.95,0.93]
pos_in_region = [0.0, -1, 1.0, 1.0]
subplot_ynorm = $
   ((pos_regions(2) - pos_regions(0)) / subplot_nhours * aspect(/device))
subpltgap_ynorm = 0.03   ; Space for hour labels of subplots, norm y distance
pos_regions(1) = pos_regions(3) - ((subplot_ynorm + subpltgap_ynorm) *nsubplots)
pos_in_region(1) = subpltgap_ynorm / (subplot_ynorm + subpltgap_ynorm)
if pos_regions(1) lt 0.02 then $
   message, /inform, 'Too many subplots (' + strim(nsubplots) + $
      ') for this length of subplot (' + strim(subplot_nhours) + ' hours)'

pos_regions = [0.05,0.05,0.95,0.95]

; Start plot

;%% repeat begin ; until hardcopy()
;%% time

; Set up subplot regions
freqx1 = freqx
if freqx eq 10 then freqx1 = 0

region_multi, /init, [1,nsubplots], pos=pos_regions, $
   title='CADI ' + h.site + " h't    " + $
   strcantime(/date,candate) + '    Day ' + string(daynum, format='(i3)') + $
   '    ' + string(h.freqs(freqx1)/1e6, format='(f6.3)') + ' MHz'

; Loop through subplots

for subplotx=0,nsubplots-1 do begin
   ; Hours on axis, including ends
   subplot_hours = $
      start_hour + subplot_nhours * subplotx + lindgen(subplot_nhours+1)
   ; Axis range (seconds)
   plot_timerange = ends(subplot_hours)*3600L

   ; Define axes of subplot without drawing

   region_multi, subplotx
   region_position, pos_in_region
   plot, /nodata, [0], [0], $
      xrange=plot_timerange, xstyle=5, $
      yrange=[0,max_height], ystyle=5

   ; Read and plot data for this subplot
   freqx1 = freqx
   freqx2 = freqx
   if freqx eq 10 then begin
     freqx1 = 0
     freqx2 = 1
   endif
   cadi = cadi_read_dir(dir=dir, timerange=candate+plot_timerange, $
      ext=file_ext, /tfh, fr=[freqx1,freqx2], error=read_error, $
      inform=keyword_set(inform))
   if cadi.ndata gt 0 then begin

      ; Find smallest rectangle containing all set pixels

      min_xt = min(cadi.data.x.t, max=max_xt)
      min_xh = min(cadi.data.x.h, max=max_xh)
      tdim = max_xt - min_xt + 1
      hdim = max_xh - min_xh + 1

      ; Create an image of this smallest rectangle and plot it

      z = 0             ; Deallocate memory
      z = make_array( tdim, hdim, /byte)
      if (size(z))(0) ne 2 then $
         ; Ensure Z is 2D even if one or both dimensions are 1
         z = reform( temporary(z), tdim, hdim )
      z( cadi.data.x.t-min_xt, cadi.data.x.h-min_xh ) = 1B
      cadi_tvft, z, $
         cadi.header.times(min_xt:max_xt) - candate, $
         cadi.header.heights(min_xh:max_xh), $
         dx=dt, dy=dh, fore_color=fore_color, back_color=back_color
      endif ; cadi.ndata gt 0

   ; Plot axes for subplot

   if subplotx eq nsubplots-1 then $
      xtitle = 'Hour (UT)' $
   else $
      xtitle = ' '
   plot, /nodata, [0], [0], $   ; Plot axes
      xrange=plot_timerange, xstyle=1, $
      xticklen=!p.ticklen / (!p.region(3)-!p.region(1)), $
      xticks=subplot_nhours, xtickv=subplot_hours*3600L, xminor=6, $
      xtickname=string(subplot_hours, format='(i2.2)'), xtitle=xtitle, $
      yrange=[0,max_height], ystyle=1, ytitle='Height (km)', yminor=5, $
      yticklen=-0.6*!p.ticklen

   endfor ; subplotx

region_multi, /reset
;%% endrep until hardcopy()

!p.noerase = 0
!p = psav
!x = xsav
!y = ysav

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
endif $;above procedure is for 4 panel day, following is for allfreq
  else begin
   ;doing allfreq
   ;print, 'doing allfreq'
   usersym, [-1.0,1.0,1.0,-1.0,-1.0],[-0.5,-0.5,0.5,0.5,-0.5],/fill   ;rect symbol for data plotting
   dbincr = floor(250/(h.nfreqs-1))
      title='CADI ' + h.site + " h't    " + $
   strcantime(/date,candate) + '    Day ' + string(daynum, format='(i3)')
   plot_timerange = timerange - candate
   plot, /nodata, [0], [0], xrange=plot_timerange, xstyle=5, title = title, $
      yrange = heightrange, ystyle=1, ytitle = 'height, km', pos = [0.08,0.08,0.98,0.94]

   for f = 0, h.nfreqs - 1 do begin
      cadi = cadi_read_dir(dir=dir, timerange=candate+plot_timerange, $
        ext=file_ext, tfh=0, fr=[f,f], error=read_error, htlimitmin = heightrange(0), htlimitmax = heightrange(1), $
        inform=keyword_set(inform))
   ;stop
   for n = 0, cadi.ndata-1 do begin
     xpos = cadi.data(n).x.t*h.dtime + plot_timerange(0)
     ypos = cadi.data(n).x.h*3.0
     rpt1 = cadi.data(n).iq(0,0)
     if rpt1 gt 127 then rpt1 = rpt1 - 256
     ipt1 = cadi.data(n).iq(1,0)
     if ipt1 gt 127 then ipt1 = ipt1 - 256
   maxpowdb = alog10(float(rpt1)^2 + float(ipt1)^2)*55.0
     ;print, maxpowdb
     oplot, [xpos], [ypos],psym = 8, $									;do the plotting
              color = 4 + dbincr*f, symsize = float(maxpowdb)/255 + 0.2
;stop
   endfor
   endfor
   ; Plot timeaxes
   ;top = (panelx eq 0)
   ;nolabels = (panelx ne npanels-1)
   ;if nolabels then $
    ;  xtitle = ' ' $
   ;else $
    ;  xtitle = ''       ; Use default
   ;!p.ticklen = pticklen / (pos(3) - pos(1))
   ;!x.charsize = charsize*0.75   ; Compensate CHARSIZE=0.8 in TIMESCALE
   timescale, plot_timerange(0), plot_timerange(1), title=xtitle, charsize = charsize, $
      top=top, nolabels=nolabels, tic_factor=-1, nodaylabels=nodaylabels
   ;!p.ticklen = pticklen


  endelse


error = !false
return

error_return:
error = !true
if not keyword_set(error_message) then error_message = ''
close, /all
region_multi, /clear
message, /inform, 'Aborted. '+error_message
xyouts, 0.3, 0.04, /norm, "Aborted h't summary plot for " + $
   datestr_in + ', ' + dir + '*.' + file_ext + $
   '!c' + error_message
return

end
