;+
; NAME:
;       CADI_SITE
; PURPOSE:
;       Returns site-specific information on CADI configuration.
; CALLING SEQUENCE:
;       Result = CADI_SITE( SITE_CODE, CANTIME )
; INPUTS:
;       SITE_CODE = Scalar string. The three-character site code. Case is
;          ignored. Valid values are: EUR GIL R_L R_B. Mandatory.
;       CANTIME = Scalar long: CANOPUS time for which configuration is required.
;          Mandatory.
; OUTPUTS:
;       Result = Structure containing information on CADI site at given time.
;          The fields of Result are:
;
;       CODE is three-letter site code.
;
;       TIME is the CANOPUS date/time to which the site description refers.
;
;       NAME is the full name of the site.
;
;       GDLON is the geodetic latitude of the site, in radians positive north.
;
;       GDLAT  is the geodetic longitude of the site, in radians positive east.
;
;       SITE.ANTENNAX(0..3) hold, for antennas E W N S, the indices into the
;       sequence of I/Q byte pairs in the data files. Nominally 0 1 2 3.
;
;       SITE.POLARITY(0..3) (integer) hold polarities for antennas E W N S:
;       1=nominal, -1=inverted. For a non-nominal pair (an overhead echo gives
;       a pi phase difference), set one to 1 and the other to -1, though in
;       practice we usually don't know which is the inverted antenna of the
;       pair.
;
;       SITE.SEPARATION(0..1) hold antenna separations in metres for pairs EW,
;       NS.
;
;       AZIMUTH is the orientation of the antenna array measured in radians
;       from true north toward east.
;
; MODIFICATION HISTORY:
;       Ian Grant, UWO: Written before 19 Oct 1994.
;       Updates to site configurations are commented in the code.
;       JMacDougall, 12June1995 added phasecorrection
;       JMD, 5may00, added Vietnam
;-

function cadi_site, site_code, cantime

on_error, 2

; Nominal settings

antennax = findgen(4)
polarity = intarr(4)+1
phasecorrectionNS = 0.0
phasecorrectionEW = 0.0
if (site_code eq 'RBL') then site_code = 'R_L'
case strupcase(site_code) of
   'ALT': begin
      code = 'ALT'
      name = 'Alert'
      gdlon = -62.3 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 82.45 * !dtor ;%%
      magdip = 86.62
      gyrofreq = 1.38
     polarity = [1, -1, 1, -1]
      separation = [20.1, 20.1]
      azimuth = 22.33 * !dtor
      end

   'CAM': begin
      code = 'CAM'
      name = 'Cambridge Bay'
      gdlon = -105.11 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 69.10 * !dtor ;%%
      magdip = 86.7
      gyrofreq = 1.66
      polarity = [1, -1, 1, -1]
      separation = [60, 60]
      azimuth = 40.0 * !dtor
      end

   'CXM': begin
      code = 'CXM'
      name = 'Cachimbo'
      gdlon = -54.95 * !dtor ;%% "geographic"
      gdlat = -9.33 * !dtor ;%%
      magdip = 4.19 ;should be -ve
      gyrofreq = 0.69
      polarity = [1, -1, 1, -1]
      separation = [21, 21]
      azimuth = 0.0 * !dtor
      phasecorrectionEW = -40.0*!dtor ; 0 deg
      phasecorrectionNS = -140.0*!dtor ; 0 deg
     end

   'DEL': begin
      code = 'DEL'
      name = 'Delaware'
      gdlon = -81 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 42.9 * !dtor ;%%
      magdip = 70.7
      gyrofreq = 1.55
      polarity = [1, -1, 1, -1]
      separation = [60, 60]
      if cantime ge cantime(2001,4,23,18,0,0) then $
         separation = [20.0, 20.0]
      if cantime ge cantime(2001,4,30,20,0,0) then $
         separation = [60.0, 60.0]
	  if cantime ge cantime(2004,7,27,0,0,0) then $
         separation = [31.0, 31.0]

     azimuth = 0.0 * !dtor
      phasecorrectionEW = -0.7853 ; -45 deg
      phasecorrectionNS = -1.1345 ; -65 deg
    if cantime ge cantime(2000,1,1,0,0,0) then begin
     phasecorrectionEW = 0*!dtor
     phasecorrectionNS = 0*!dtor
    endif
    if cantime ge cantime(2001,8,1,0,0,0) then begin
     phasecorrectionEW = 20*!dtor
     phasecorrectionNS = 90*!dtor
    endif
    if cantime ge cantime(2004,8,1,0,0,0) then begin
     phasecorrectionEW = -46*!dtor
     phasecorrectionNS = -10*!dtor
    endif
   end

    'EUR': begin
      code = 'EUR'
      name = 'Eureka'
      gdlon = -85.92 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 79.99 * !dtor ;%%
      magdip = 88.1
      gyrofreq = 1.40
      polarity = [1, -1, 1, -1]
      separation = [83.0, 85.0]
      phasecorrectionNS = 0.0 ;0 deg  ;phase corrections before Dec 1996
      phasecorrectionEW = 0.0
      azimuth = 15.3 * !dtor ;previously to April 14/96 14.0 used
      if cantime ge cantime(1993,7,17,0,0,0) then $
         separation = [60.0, 60.0]
      if cantime ge cantime(2000,7,22,0,0,0) then $
         separation = [40.0, 40.0]
     if cantime ge cantime(1996,12,1,0,0,0) then begin ; phasecorrection changed
      phasecorrectionNS = 0.18 ; 10.5 deg
      phasecorrectionEW = 0.69 ; 40 deg
     endif
     if cantime ge cantime(1997,3,1,0,0,0) then begin ; phasecorrection changed
      phasecorrectionNS = 0.46 ; 26 deg
      phasecorrectionEW = 0.83 ; 48 deg
     endif
     if cantime ge cantime(1999,1,1,0,0,0) then begin ; phasecorrection changed
      phasecorrectionNS = 0.70 ; 40 deg
      phasecorrectionEW = 1.23 ; 70 deg
     endif
     if cantime ge cantime(2000,12,1,0,0,0) then begin ; phasecorrection changed
      phasecorrectionNS = 0.35 ; 20 deg
      phasecorrectionEW = 1.57 ; 90 deg
     endif
     if cantime ge cantime(2002,1,1,0,0,0) then begin
      phasecorrectionNS = 0.70 ; 40 deg
      phasecorrectionEW = 1.57 ; 90 deg
     endif
     end

   'FOR': begin
      code = 'FOR'
      name = 'Fortaleza'
      gdlon = 322 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = -4 * !dtor ;%%
      magdip = 11.1 ;should be -ve??
      gyrofreq = 0.64
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionEW = 0 ;deg
      phasecorrectionNS = 10.0*!dtor ;-30.0*!dtor ;deg
      end

   'GIL': begin
      code = 'GIL'
      name = 'Gillam'
      gdlon = 0.0 * !dtor ;%%
      gdlat = 0.0 * !dtor ;%%
      polarity = [1, -1, 1, -1]
      separation = [20.0, 20.0]
      azimuth = (17.0+(44/60.0)) * !dtor
      end

'LON': begin
      code = 'LON'
      name = 'London'
      gdlon = -81.28 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 43.00 * !dtor ;%%
      magdip = 70.7
      gyrofreq = 1.55
      polarity = [1, -1, 1, -1]
      separation = [20, 20]
      azimuth = 0.0 * !dtor
      phasecorrectionEW = 0 * !dtor ;
      phasecorrectionNS = 0 * !dtor ;
    end

   'NAT': begin
      code = 'NAT'
      name = 'Natal'
      gdlon = -35.200 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = -5.7833 * !dtor ;%%
      magdip = 17.18 ;should be -ve??
      gyrofreq = 0.63
     polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionNS = -90.0*!dtor; 0 deg
      phasecorrectionEW = -60.0*!dtor; 0 deg
      end

'OTT': begin
      code = 'OTT'
      name = 'Cape Race'
      gdlon = -81 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 43 * !dtor ;%%
      polarity = [1, -1, 1, -1]
      separation = [60, 60]

     azimuth = 0.0 * !dtor
      phasecorrectionEW = 0.0
      phasecorrectionNS = 0.0
  end

'P_R': begin
      code = 'P_R'
      name = 'Parit Raja'
      gdlon = 103 * !dtor ;%% 2-digit "geographic" from handout
      gdlat = 2 * !dtor ;%%
      magdip = 14.3 ;should be -ve??
      gyrofreq = 1.17
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionEW = 0.0*!dtor ;deg
      phasecorrectionNS = 0.0*!dtor ;deg
      end

 'P_S': begin
      code = 'P_S'
      name = 'Park Site'
      gdlon = -106* !dtor ;%% 2-digit "geographic" from handout
      gdlat = 52 * !dtor ;%%
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionNS = 0.0; 0 deg
      phasecorrectionEW = 0.0; 0 deg
      end

    'R_L': begin
      code = 'R_L'
      name = 'Rabbit Lake'
      gdlon = 0.0 * !dtor ;%%
      gdlat = 0.0 * !dtor ;%%
      magdip = 75.0
      gyrofreq = 1.42
      polarity = [1, 1, 1, -1] ; is this still correct ???
      separation = [75.0, 75.0]
      azimuth = 24.0 * !dtor
      if cantime ge cantime(1993,1,1,0,0,0) then begin ;%% 93,8,14? p.32, Ask Willy
         separation = [60.0, 60.0]
         polarity = [1,-1,1,-1]  ;??
       endif
     if cantime ge cantime(1994,9,21,0,0,0) then begin ; antenna spacing changed
         polarity = [1,-1, 1,-1]
         separation = [20.0, 20.0]
     endif
     phasecorrectionEW = +0.78 ; +45 deg  ;0.873 ; 50 deg
     phasecorrectionNS = 0.39   ;+0.78 ; +45 deg ;0.175 ; 10 deg
     end

    'R_B': begin
      code = 'R_B'
      name = 'Resolute Bay'
      gdlon = (360.0 - (95.0 + 5.0/3600.0)) * !dtor
      gdlat = (74.0 + 44.0/60.0 + 43.0/3600.0) * !dtor
      magdip = 88.0
      gyrofreq = 1.42
     if cantime lt cantime(1994,7,25,15,0,0) then antennax = [2, 3, 0, 1]
;      if cantime ge cantime(1994,7,25,15,0,0) then  $
;           antennax = findgen(4)
      polarity = [1, -1, 1, -1]
      separation = [60.0, 60.0]
      azimuth = 24.0 * !dtor ; Changed entry from 0.0 to 24.0 deg on 5 May 1994
     ; phasecorrectionNS = -1.25   ;72 deg (->rad)correction for antenna 4 wrt antenna 3
      phasecorrectionEW = -0.73   ;42 deg (->rad)correction for antenna 2 wrt antenna 1
      if cantime gt cantime(1995,10,01,0,0,0) then begin
      phasecorrectionEW = -0.35 ; -20 deg
      phasecorrectionNS = 0.35 ; 20 deg
      endif
	  end

    'RAN': begin
      code = 'RAN'
      name = 'Rankin Inlet'
      gdlon = (360.0 - (92.0 + 7.0/60.0)) * !dtor
      gdlat = (62.0 + 49.0/60.0) * !dtor
      magdip = 83.5
      gyrofreq = 1.46
      polarity = [1, -1, 1, -1]
      separation = [20.0, 20.0]
      azimuth = 0.0 * !dtor ;
      phasecorrectionEW = 0.0 ; 0 deg
      phasecorrectionNS = 0.0 ; 0 deg
      end


 'SJC': begin
      code = 'SJC'
      name = 'Sao Jose dos Campos'
      gdlon = -70* !dtor ;%% 2-digit "geographic" from handout
      gdlat =  -25* !dtor ;%%
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionNS = 0.0; 0 deg
      phasecorrectionEW = 0.0; 0 deg
      end

'SIL': begin
      code = 'SIL'
      name = 'SIL'
      gdlon = -106* !dtor ;%% 2-digit "geographic" from handout
      gdlat = 52 * !dtor ;%%
      magdip = 76.0
      gyrofreq = 1.64
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionNS = 0.0; 0 deg
      phasecorrectionEW = 0.0; 0 deg
      end



  'HCM': begin
      code = 'HCM'
      name = 'HoChiMinhVietnam'
      gdlon = 180.0 * !dtor ;%% 2-digit "geographic"
      gdlat = 0.0 * !dtor ;%%
      polarity = [1, -1, 1, -1]
      separation = [21.0, 21.0]
      azimuth = 0.0 * !dtor
      phasecorrectionNS = 0.0
      phasecorrectionEW = 0.0
      end


   else: message, 'Unrecognized site code'
   endcase

return, $
   {code:       code, $
    time:       cantime, $
    name:       name, $
    gdlon:      gdlon, $
    gdlat:      gdlat, $
    magdip:		magdip, $
    gyrofreq:	gyrofreq, $
    antennax:   antennax, $
    polarity:   polarity, $
    separation: separation, $
    azimuth:    azimuth, $
    phasecorrectionEW:   phasecorrectionEW,  $
    phasecorrectionNS:   phasecorrectionNS}

end
