;+
; NAME:
;	BIN
; PURPOSE:
;	Classify elements of an array into bins according to a vector of levels.
;	E.g. assign color indices to values in an array according to a scale.
; CALLING SEQUENCE:
;	Result = BIN (A, LEVELS, CLASSES, ELSE_CLASS)
; INPUTS:
;	A = Input array of any dimensions and any numeric type.
;	LEVELS = Vector of values at boundaries of bins.
;		Must have one more element than CLASSES.
;	CLASSES = Vector holding value to be returned for each bin.
; OPTIONAL INPUT PARAMETERS:
;	ELSE_CLASS = Value to be returned if input value is outside range
;		of LEVELS. Default is -1.
; OUTPUTS:
;	Result = array of type of CLASSES and dimensions of A.
;	Result(i) = { CLASSES(0)   if LEVELS(0)   <= A(i) < LEVELS(1)
;                   { ...
;	            { CLASSES(n-1) if LEVELS(n-1) <= A(i) < LEVELS(n)
;	            { ELSE_CLASS   if A(i) < LEVELS(0) and A(i) >= LEVELS(n)
; PROCEDURE:
;	Uses WHERE.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC 15 May 1991: IDL2.
;	IFG, 4 Jan 1993: Added ON_ERROR.
;	IFG, 30 Apr 1993: Changed to handle A a scalar.
;-

function bin, a, levels, classes, else_class

on_error, 2
if n_elements(classes) lt 1 then $
   message, 'CLASSES must have at least one element'
if n_elements(levels) ne (n_elements(classes)+1) then $
   message, 'LEVELS must have exactly one more element than CLASSES'
if n_params() lt 4 then else_class = -1

sa = size(a)
ndims_a = sa(0)
if ndims_a eq 0 then $
   b = else_class $	; A is a scalar, so B must be made a scalar
else $
   b = make_array(type=type(classes), dimension=sa(1:sa(0)), value=else_class)
for classx = 0,n_elements(classes)-1 do begin
   w = where( (a ge levels(classx)) and (a lt levels(classx+1)) , wcount)
   if wcount gt 0 then $
      b(w) = classes(classx)
   endfor
return, b
end
