;+
; NAME:
;	CONTOUR_PREFILL
; PURPOSE:
;	Modify Z (2D), X and Y (1D) for filled contours including open ones.
; CALLING SEQUENCE:
;	CONTOUR_PREFILL, Z, X, Y, ZZ, XX, YY, $
;	   LEVELS, COLORS, FILL_LEVELS, FILL_COLORS
; INPUTS:
;	Z = 2D array of dimensions (N_ELEMENTS(X), N_ELEMENTS(Y)). Data to be 
;	   contoured.
;	X = vector of abscissae of points to be contoured.
;	Y = vector of ordinates of points to be contoured.
;	LEVELS = vector of contour levels (before filling).
;	COLORS = vector of indices into color table (before filling).
; OUTPUTS:
;	ZZ, XX, YY, FILL_LEVELS, FILL_COLORS: Similar to corresponding inputs. 
;	   These are to be used as arguments in a subsequent call to CONTOUR.
; Example:
;	; Given Z, X, Y, LEVELS, COLORS
;	contour_prefill, z, x, y, zz,xx,yy, $
;	   levels, colors, fill_levels, fill_colors
;	contour, zz, xx, yy, path='cpaths.dat', $
;	   levels=fill_levels, /follow, ystyle=5, /overplot
;	polycontour, 'cpaths.dat', /delete, color=fill_colors
; RESTRICTIONS:
;	X and Y must have only one dimension.
; PROCEDURE:
;	Put a border of non-contoured values around the 2D array, as suggested
;	for POLYCONTOUR, but plot the border very narrow and with its outer
;	edge at the desired position of the 2D array. 
;
;	Creation of FILL_LEVELS and FILL_COLORS is needed to ensure background 
;	(i.e. region outside closed contours) is filled with COLORS(0), because
;	POLYCONTOUR says it will fill the background but won't.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 22 Oct 1991: Written in IDL2.
;	IFG, 22 Dec 1992: Added ON_ERROR.
;	GRANT, 16 Sep 1993: Changed BIG_VALUE from 1e20 to a value comparable 
;	   to abs(min(Z,LEVELS)), to avoid divide by zero errors. Changed
;	   first element of FILL_LEVELS to use min(Z,LEVELS), in case LEVELS 
;	   has elements < min(Z).
;	      FILL_COLORS no longer has dummy first element prepended - this
;	   seems necessitated by change from POLYCONTOUR to CONTOUR, /FILL.
; Possible enhancements:
;	A tidier way to handle the open contours might be to produce a version 
;	of POLYCONTOUR.PRO which clips the image, perhaps by replacing polygon 
;	vertices outside the clipping rectangle with points on the edge of the
;	rectangle.
;-

pro contour_prefill, z, x, y, zz, xx, yy, $
   levels, colors, fill_levels, fill_colors

on_error, 2
minz = min(z)
minzlevels = min([ minz, min(levels) ])
big_value = abs(minzlevels) + 1.0

nx = n_elements(x)
dx = 0.0001 * (x(nx-1) - x(0))
xx = [x(0), x(0)+dx, x(1:nx-2), x(nx-1)-dx, x(nx-1)]

ny = n_elements(y)
dy = 0.0001 * (y(ny-1) - y(0))
yy = [y(0), y(0)+dy, y(1:ny-2), y(ny-1)-dy, y(ny-1)]

zz = replicate(minz-big_value, nx+2, ny+2)
zz(1,1) = z

fill_levels = [minzlevels-0.5*big_value, levels]
fill_colors = colors

return
end
