;+
; NAME:
;	EDGC
; PURPOSE:
;	Convert EDFL coordinates to geocentric, near the Earth's surface.
; CALLING SEQUENCE:
;	edgc, edlat, edlon, alt, gclat, gclon, gcrkm
; INPUTS:
;	EDLAT = EDFL latitude in radians.
;	EDLON = EDFL longitude in radians.
;	ALT   = Altitude (geodetic) above MSL in km.
;	All inputs are FLOAT, and should be either all scalars or all vectors.
; OUTPUTS:
;	GCLAT = Geocentric latitude in radians.
;	GCLON = Geocentric longitude in radians.
;	GCRKM = Geocentric distance in km.
;	All outputs are FLOAT, and are scalars or vectors according to input.
; PROCEDURE:
;	Calls CANECDIP twice, with call to GCGD in between.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 19 Feb 1992: Written in IDL2.
;       IFG, 24 Jul 1992: Added ON_ERROR.
;-

pro edgc, edlat, edlon, alt, gclat, gclon, gcrkm

on_error, 2
if n_params() ne 6 then message, 'You must provide exactly 6 parameters'

flag = 1	; CANECDIP EDFL to GC
re = 6362.9	; km at BACK. Nominal for initial guess. Exact value not
                ; important, but this gives alt error < 1 m. 6371.2 gives ~5 m.
barsalt = 107.0
elat = array(edlat)
elon = array(edlon)
ealt = array(alt)
n = n_elements(elat)
gcrkm = re + ealt
gclat = fltarr(n)
gclon = fltarr(n)
erkm = fltarr(n)

canecdip, flag, gclat, gclon, gcrkm, elat, elon, erkm	; First try
gcgd, gclat, gcrkm, gdlat, gdalt		; Find error in geodetic alt
gcrkm = re + ealt - (gdalt - ealt)		; Correct guess by alt error
canecdip, flag, gclat, gclon, gcrkm, elat, elon, erkm	; Second try

if not isarray(edlat) then begin
   gclat = gclat(0)
   gclon = gclon(0)
   gcrkm = gcrkm(0)
   endif

; Test: expect gdalt = alt
;
; gcgd, gclat, gcrkm, gdlat, gdalt
; gdlon = gclon
; print, 'gclat, gclon, gcrkm = ', gclat(0)*!radeg, gclon(0)*!radeg, gcrkm(0) 
; print, 'gdlat, gdlon, gdalt = ', gdlat(0)*!radeg, gdlon(0)*!radeg, gdalt(0)
; print, 'nominal re, actual re, erkm = ', re, gcrkm(0)-gdalt(0), erkm(0)

return
end
