;+
; NAME:
;	ENTER
; PURPOSE:
;	Enter a number, string, expression or command.
;	Allow entry and execution of IDL expressions, IDL commands,
;	DCL commands, Quit command, Backup command, as well as simple number
;	or text entry.
; CATEGORY:
;	Programming utility.
; CALLING SEQUENCE:
;	VALUE = ENTER(TYP, PROMPT, DEFAULT, EXIT_FLAG, HELP_TEXT)
; INPUTS:
;   TYP = Data type to return.  Allowed types are:
;      'STRING' = string, verbatim (no quotes)			; IFG
;      'STREXP' = string expression. Literal strings must be delimited by ".
;         (e.g. string in quotes, variable name, string array)	; IFG
;      'BYTE' = byte
;      'INTEGER' = integer
;      'LONG' = long integer
;      'FLOAT' = float
;      'DOUBLE' = double float
;   PROMPT = prompt string.
;   DEFAULT = default value.
; OPTIONAL INPUT PARAMETERS:
;   HELP_TEXT = string (string array for multi-lines) to display if ? entered.
; OUTPUTS:
;   VALUE = returned value of desired type.
; OPTIONAL OUTPUT PARAMETERS:
;   EXIT_FLAG = exit status flag.  Valid values are:
;      0 = normal exit.
;      1 = quit on return.
;     -1 = backup to last entry.
;      11 = invalid data type.
;      12 = default value cannot be converted to desired data type.
; RESTRICTIONS:
;	 The following commands are allowed as ENTER entries:
;   Value requested.
;   Null entry. Gives default value (for calling error may return error flag).
;   Expressions. processes given IDL expression.
;   Q. Returns quit value (1) in exit flag.
;   B. Returns backup value (-1) in exit flag.
;   IDL statment. Executes given IDL statement.
;   $ statement. Executes given DCL statement.
; MODIFICATION HISTORY:
;	R. Sterner.  17 Oct, 1986.
;	Ian Grant, HIA/NRCC: Revised to run under IDL2.
;	IFG, 21 Nov 1991: Added entry type STREXP.
;	IFG, 22 Dec 1992: Added ON_ERROR.
;-

	FUNCTION ENTER, TYP, PROMPT, DEF, EXIT_FLAG, HELP_TEXT

	ON_ERROR, 2
	IF N_PARAMS(0) LT 3 THEN BEGIN	
	  PRINT,'Error in ENTER: must call with at least 3 args.'
	  RETURN, 0
	ENDIF

	IF N_PARAMS(0) LT 5 THEN HELP_TEXT = ' '	; help text.
	HELP_TEXT = ARRAY(HELP_TEXT)			; force to be array.
	NHELP = N_ELEMENTS(HELP_TEXT)			; number of lines.

	TYP = STRUPCASE(TYP)				; force type upper.

;------------  get entry  ---------------------
LOOP:	F = "($,'" + PROMPT + "')"	; Build print format string. %% IFG
	PRINT,FORMAT=F			; print prompt (no CR after). %% IFG
	TXT = ''			; make TXT be a text string.
	READ, TXT			; read TXT.
;------------  end of get entry  --------------

;------------  null entry  --------------------
	IF TXT EQ '' THEN BEGIN		; try to return default.
	  ON_IOERROR, DEFERR		; catch conversion errors.
	  DUMMY = CHECK_MATH(1,1)	; %% IFG ON_MATHERROR, DEFERR
	  CASE TYP OF			; try to convert to desired type.
'STRING':   X = STRTRIM(DEF,2)
'STREXP':   X = STRTRIM(DEF,2)
'BYTE':	    BEGIN
	      X = FIX(DEF)
	      IF (X GT 255) OR (X LT 0) THEN GOTO, DEFERR
	      X = BYTE(X)
	    END
'INTEGER':  X = FIX(DEF)
'LONG':	    X = LONG(DEF)
'FLOAT':    X = FLOAT(DEF)
'DOUBLE':   X = DOUBLE(DEF)
ELSE:	    BEGIN
	     PRINT,'Error in ENTER: unknown data type.'
	     EXIT_FLAG = 11
	     RETURN, 0
	    END
	  ENDCASE
	  IF CHECK_MATH(0,0) NE 0 THEN GOTO, DEFERR	;%% IFG
	  EXIT_FLAG = 0			; no error.
	  RETURN, X			; return converted default.
DEFERR:	  BEGIN				; conversion error on DEF.
	    PRINT,'Error in ENTER: cannot convert given default to '+TYP+' data type.'
	    EXIT_FLAG = 12
	    RETURN, 0
	  END
	ENDIF
;------------  end of null entry  --------------------

;------------  check for ?  ---------------------------
	IF GETWRD(TXT, 0) EQ '?' THEN BEGIN
	  PRINT,' '
	  FOR I = 0, NHELP-1 DO PRINT,HELP_TEXT(I)
	  TXT = 'The desired data type for this entry is ' + TYP + '.'
	  PRINT,'The following are allowed as entries:'
	  PRINT,'A value as requested.  ' + TXT
	  PRINT,'Null entry.            Pressing RETURN gives the default value.'
	  PRINT,'Expression.            Uses the given IDL expression, like SQRT(2)'
	  PRINT,'Q                      Quits.'
	  PRINT,'B                      Backs up to last entry.'
	  PRINT,'IDL statment.          Executes the given IDL statement.'
	  PRINT,'$ statement.           Executes the given DCL statement.'
	  PRINT,' '
	  GOTO, LOOP
	ENDIF
;-----------  end of ?  -----------------------------


;------------  check for quit command  ---------------
	IF STRUPCASE(GETWRD(TXT,0)) EQ 'Q' THEN BEGIN
	  EXIT_FLAG = 1
	  RETURN, 0
	ENDIF
;------------  end of quit command  ---------------


;------------  check for backup command  ---------------
	IF STRUPCASE(GETWRD(TXT,0)) EQ 'B' THEN BEGIN
	  EXIT_FLAG = -1
	  RETURN, 0
	ENDIF
;------------  end of quit command  ---------------


;------------  check for idl command  --------
	IF STRUPCASE(GETWRD(TXT,0)) EQ 'IDL' THEN BEGIN
	  I = EXECUTE(GETWRD(TXT,-1))
	  GOTO, LOOP
	ENDIF
;-----------  end of idl command  ---------------------


;------------  check for dcl  command  --------
	IF STRSUB(TXT,0,0) EQ '$' THEN BEGIN
	  I = EXECUTE("$ "+STRSUB(TXT,1,999))	; %% IFG
	  GOTO, LOOP
	ENDIF
;-----------  end of DCL command  ---------------------


;------------  expression or value  -------------------
	IF TYP EQ 'STRING' THEN BEGIN	; put literal string entry in ' '
	  TXT = "IN = '" + TXT + "'"
	ENDIF ELSE BEGIN		; but not other entries.
	  TXT = "IN = " + TXT
	ENDELSE
	I = EXECUTE(TXT)
	IF I NE 1 THEN BEGIN
	  PRINT,' '
	  PRINT,'Enter ? for help.'
	  PRINT,' '
	  GOTO, LOOP
	ENDIF
	ON_IOERROR, EXERR		; catch conversion errors.
	DUMMY = CHECK_MATH(1,1)		; %% IFG ON_MATHERROR, EXERR
	CASE TYP OF			; try to convert to desired type.
'STRING': X = STRTRIM(IN,2)
'STREXP': X = STRTRIM(IN,2)
'BYTE':	  BEGIN
	    X = FIX(IN)
	    IF (X GT 255) OR (X LT 0) THEN GOTO, EXERR
	    X = BYTE(X)
	  END
'INTEGER':X = FIX(IN)
'LONG':	  X = LONG(IN)
'FLOAT':  X = FLOAT(IN)
'DOUBLE': X = DOUBLE(IN)
ELSE:	  BEGIN
	    PRINT,'Error in ENTER: unknown data type.'
	    EXIT_FLAG = 11
	    RETURN, 0
	  END
	ENDCASE
	IF CHECK_MATH(0,0) NE 0 THEN GOTO, EXERR	; %% IFG
	EXIT_FLAG = 0			; no error.
	RETURN, X			; return converted entry.
EXERR:	BEGIN				; conversion error on IN.
	  PRINT,'Error in ENTER: cannot convert given entry to '+TYP+' data type.'
	  GOTO, LOOP
	END
;------------  end of expression or value  ------------

	END
