;+
; NAME:
;	FILTER
; PURPOSE:
;	Butterworth filter a time series.
;	Pads each end with 2880 copies of the respective end point.
;	Interpolates across missing data, flagged by values GE 99999.
; CALLING SEQUENCE:
;	FILTER, FTYPE_IN, N_IN, SAMPINT_IN, FLP_IN, FHP_IN, NF_IN, XX
; INPUTS:
;	ftype     LONG   Filter type: 1=highpass, 2=lowpass, 3=bandpass
;	                 Default is causal filter.
;	                 ftype negative selects acausal filter with zero phase
;	                 shift.
;	n         LONG   Number of points in time series XX.
;	sampint   FLOAT  Sampling interval in seconds.
;	flp       FLOAT  Maximum frequency of passband, or cutoff for lowpass.
;	                 Hertz.
;	fhp       FLOAT  Minimum frequency of passband, or cutoff for highpass.
;	                 Hertz.
;	nf        LONG   Number of poles in Butterworth filter. Usually 10L.
;	xx        FLOAT  Data vector. In/out.
; KEYWORD:
;	MAX_VALUE = FLOAT scalar input. Elements of XX which are GE MAX_VALUE
;		are replaced by linear interpolation before filtering and set
;		to MAX_VALUE after. Default is 99999.0.
; OUTPUTS:
;	xx        FLOAT  Data vector. In/out.
; PROCEDURE:
;	FORTRAN routine does causal filtering.
;	Acausal filter implemented by applying causal filter, time reversing, 
;	filtering again, time reversing again.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRC, 20 Mar 1991. IDL2.
;		From [TRICKEY.GRAPHICS]FILTER.PRO (IDL2) with negative FTYPE
;		implemented as in Terry Hughes' IDL1 FILTER.PRO.
;	IFG 27 July 1992: Added MISSING keyword.
;		Changed CALL_VMS to CALL_EXTERNAL. Added ON_ERROR.
;	IFG 15 Dec 1992: No mods. Noted that differences between this routine 
;		wrt FILTER in MARIA_IDL.TLB are:
;		- CALL_EXTERNAL instead of CALL_VMS
;		- MISSING as keyword instead of constant
;		- Test for presence of bad points with count from WHERE rather 
;		   than WHERE(...)(0) EQ -1.
;	IFG  4 Feb 1993: Renamed MISSING to MAX_VALUE.
;-

PRO FILTER, FTYPE_IN, N_IN, SAMPINT_IN, FLP_IN, FHP_IN, NF_IN, XX, $
   	MAX_VALUE=MAX_VALUE	

ON_ERROR, 2

FTYPE   = LONG(FTYPE_IN)
N       = LONG(N_IN)
SAMPINT = FLOAT(SAMPINT_IN)
FLO     = FLOAT(FLP_IN)
FHI     = FLOAT(FHP_IN)
NF      = LONG(NF_IN)
XX      = FLOAT(XX)
if not keyword_set(max_value) then max_value = 99999.0	; IFG

;      +-------------------------------------------------------------------+
;      |DO NOT CHANGE CASE OF FOLLOWING LINE !!!                           |
;      +-------------------------------------------------------------------+
if trnlog('BUTTER$EXE',logname) ne 1 then $
   SETLOG,'BUTTER$EXE','MAR$IDL$DAN:IDL_BETABUTTER.EXE'
;      +-------------------------------------------------------------------+
;      |DO NOT CHANGE CASE OF PRECEDING LINE !!!                           |
;      +-------------------------------------------------------------------+

;      +-------------------------------------------------------------------+
;      |Set up prefix and suffix data to account for impulse response      |
;      +-------------------------------------------------------------------+
pref=replicate(xx(0),2880)
suff=replicate(xx(n-1),2880)
x=[pref,xx,suff]
nn=n_elements(x)
aftype=abs(ftype)

yy=dblarr(nn) & zz=yy

fill=max_value
ibad=where(x ge max_value-9.0, nbad)
if nbad gt 0 then begin
   print,'Interpolating across fills, before filtering'
   x(ibad)=fill
   fixgaps,x,fill,1
endif

if ((aftype eq 2) or (aftype eq 3)) then begin
   fftype=2l
   istat=call_external('BUTTER$EXE','BUTTER',FFTYPE,NN,SAMPINT,FLO,NF,X,YY,ZZ)
   if ftype lt 0 then begin
      x=reverse(x)
      istat= $
         call_external('BUTTER$EXE','BUTTER',FFTYPE,NN,SAMPINT,FLO,NF,X,YY,ZZ)
      x=reverse(x)
   endif
endif

if ((aftype eq 1) or (aftype eq 3)) then begin
   fftype=1l
   istat=call_external('BUTTER$EXE','BUTTER',FFTYPE,NN,SAMPINT,FHI,NF,X,YY,ZZ)
   if ftype lt 0 then begin
      x=reverse(x)
      istat=$
         call_external('BUTTER$EXE','BUTTER',FFTYPE,NN,SAMPINT,FHI,NF,X,YY,ZZ)
      x=reverse(x)
   endif
endif

if nbad gt 0 then x(ibad)=fill
xx=x(2880:nn-2881)
x=0 & yy=0 & zz=0
return
END
