;+
; NAME:
;    FILTER_FUNC
; PURPOSE:
;    Butterworth filter one or more time series.
;    See procedure FILTER for details.
; CALLING SEQUENCE:
;    Result = FILTER_FUNC( X [,DTIME=DTIME] )
; INPUTS:
;    X       FLOAT  Data vector(s). If X has more than one dimension, then the 
;                   first dimension is treated as the time index, while the
;                   remaining dimensions index different time series.
;    DTIME   FLOAT  Sampling interval. Default is 1.0.
; KEYWORD PARAMETERS:
;    FLP     FLOAT  Maximum frequency of passband, or cutoff for lowpass. Hertz.
;    FHP     FLOAT  Minimum frequency of passband, or cutoff for highpass. Hz.
;
;    Units of FLP and FHP are (Units of DTIME) ^ -1.
;    At least one of FLP and FHP must be set.
;    Filter type is determined by FLP and FHP:
;       FLP present and nonzero            Lowpass
;       FHP present and nonzero            Highpass
;       FLP and FHP present and nonzero    Bandpass
;
;    NOZEROPHASE  Integer. If present and non-zero a causal filter is used,
;                 otherwise an acausal filter with zero phase shift is used.
;    NPOLES  LONG   Number of poles in Butterworth filter. Usually 10L.
;    STR     STRING output. Description of filter type and cut-offs (no units).
;    MAX_VALUE FLOAT scalar input. Elements of XX which are GE MAX_VALUE
;               are replaced by linear interpolation before filtering and set
;               to MAX_VALUE after. Default is 99999.0.
; OUTPUTS:
;    Result    FLOAT  Array containing filtered time series. Has same 
;                     dimensions as X.
; PROCEDURE:
;    Calls procedure FILTER.
; MODIFICATION HISTORY:
;    Ian Grant, HIA/NRCC, 21 Mar 1991. IDL2.
;    IFG 14 Jan 1992: Overhauled parameters. New keywords FLP, FHP, NOZEROPHASE.
;    IFG 23 Jan 1992: Extended to multiple time series (X with >1 dimension).
;    IFG 20 Jul 1992: Added STR output keyword.
;    IFG 27 Jul 1992: Added MISSING input keyword.
;       Changed positional parameter SAMPINT to keyword DTIME. Added ON_ERROR.
;    IFG  4 Feb 1993: Renamed MISSING to MAX_VALUE.
;-

function filter_func, x, dtime=dtime, $
   flp=flp, fhp=fhp, nozerophase=nozerophase, npoles=npoles, str=str, $
   max_value=max_value

on_error, 2

if n_params() ne 1 then message, 'Must provide exactly 1 parameter.'
if not keyword_set (dtime) then dtime = 1.0
lp = keyword_set(flp)
hp = keyword_set(fhp)
case !true of
   not lp and     hp: begin
                      ftype = 1
                      str = 'Highpass ' + strim_float(fhp)
                      end
       lp and not hp: begin
                      ftype = 2
                      str = 'Lowpass ' + strim_float(flp)
                      end
       lp and     hp: begin
                      ftype = 3
                      str = 'Bandpass ' + strim_float(fhp)+'-'+strim_float(flp)
                      end
   not lp and not hp: message, $
                      'At least one of the keywords FLP and FHP must be nonzero'
   endcase
if not lp then flp = 0.0
if not hp then fhp = 0.0

if not keyword_set(nozerophase) then ftype = -ftype
if not keyword_set(npoles)      then npoles = 10L
if not keyword_set(max_value)   then max_value = 0.0  ; Let FILTER set MAX_VALUE

sx = size(x)
ndims = sx(0)
if ndims eq 0 then message, 'X must have at least 1 dimension'
n = sx(1)		; Number of points per function

nfuncs = 1		; Number of functions
if ndims gt 1 then $
   for dx=2,ndims do $
      nfuncs = nfuncs*sx(dx)

xout = reform(x, n, nfuncs)		; Treat all dims after first as one
for funcx=0,nfuncs-1 do begin

   ; Filter each function

   xx = xout(*,funcx)
   filter, ftype, n, dtime, flp, fhp, npoles, xx, max_value=max_value
   xout(0,funcx) = xx
   endfor
xout = reform(xout, sx(1:ndims))		; Restore to original dimensions

return, xout
end
