;+
; NAME:
;	MOVING_WINDOW
; PURPOSE:
;	Return segments of multiple timeseries centred on given times.
; CALLING SEQUENCE:
;	Result = MOVING_WINDOW(X, DATA_TIMES, WINDOW_TIMES, LEN_WINDOW)
; INPUTS:
;	X            = Array of time series data. First dimension is number
;		of times. Other dimensions index different time series. FLOAT.
;	DATA_TIMES   = Vector of times for points in DATA. Length must equal
;		first dimension of X.
;		If INDEX is not set, DATA_TIMES must increase with a uniform
;		increment and be INT or LONG.
;		If INDEX is set, DATA_TIMES is ignored but a dummy parameter
;		must be used to hold its place.
;	WINDOW_TIMES = Vector of times at which the returned segments will be 
;		centred. Must be a subset of DATA_TIMES. INT/LONG.
;		If INDEX is set, then WINDOW_TIMES is a vector of indices into 
;		DATA_TIMES.
;	LEN_WINDOW   = Length of the window. Units are the same as those of 
;		WINDOW_TIMES. Must be an even multiple of the DATA_TIMES
;		increment. INT/LONG.
;		If INDEX is set, then LEN_WINDOW is the number of steps along 
;		DATA_TIMES and must be even.
;	INDEX          If INDEX is set (present and nonzero), WINDOW_TIMES and
;		LEN_WINDOW are interpreted to refer to the subscripts of
;		DATA_TIMES.
; OUTPUT:
;	Result = FLOAT array. If X has dimensions (ntimes,N1,...,Nn), then 
;		Result has dimensions (npoints,ntimes,N1,...,Nn), where
;		npoints = (LEN_WINDOW / (DATA_TIMES increment)) + 1.
;		Result(*,i,...) is a timeseries of length LEN_WINDOW centred
;		on time WINDOW_TIMES(i) (including both end points).
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 22 Apr 1992: Written in IDL2.
;	IFG, 23 Dec 1992: Added ON_ERROR.
;	GRANT,  7 Jan 1994: Added keyword INDEX.
;-

function moving_window, x, data_times, window_times, len_window, index=index

on_error, 2

; Check arguments and form HALFW

if n_params() ne 4 then message, 'Must give exactly 4 parameters'
sx = size(x)
ndims = sx(0)
if ndims eq 0 then message, 'X must have at least 1 dimension'
ntimes = sx(1)		; Number of points per time series

nfuncs = 1		; Number of time series
; List of dimensions not indexing time or window, and a trailing dimension of 1
other_dims = [1]
if ndims gt 1 then begin
   otherdims = [sx(2:ndims), otherdims]
   for dx=2,ndims do $
      nfuncs = nfuncs*sx(dx)
   endif

if not keyword_set(index) then begin ;%% 930107
   ; DELTA_TIME is only used if INDEX is not set
   if ntimes ne n_elements(data_times) then message, $
      'Number of elements in DATA_TIMES must equal first dimension of X'
   delta_times = (data_times - shift(data_times,1)) (1:ntimes-1)
   delta_time = delta_times(0)
   dummy = where(delta_times ne delta_time, wcount)
   if (wcount gt 0) or (delta_time le 0) then $
      message, 'DATA_TIMES are not uniformly increasing'
   endif ; not index

nwindows = n_elements(window_times)

; Define and check NPTS_WINDOW and HALFW

if keyword_set(index) then begin
   npts_window = len_window+1	; LEN_WINDOW is *length* i.e. number of steps
   if (npts_window mod 2) ne 1 then $
      message, 'LEN_WINDOW is not even'
   halfw = len_window / 2
   if ((window_times(0) - halfw) lt 0) or $
      ((window_times(nwindows-1) + halfw) gt ntimes-1) then $
      message, 'WINDOW_TIMES convolved with LEN_WINDOW is longer than time '+ $
         'dimension of X'
   endif $
else begin
   len_window_pts = len_window / delta_time
   npts_window = len_window_pts+1
   if (len_window ne len_window_pts*delta_time) or ((len_window_pts mod 2) ne 0) $
      then $
      message, 'LEN_WINDOW is not an even multiple of DATA_TIMES spacing'
   halfw = len_window_pts / 2
   if ((window_times(0) - halfw*delta_time) lt data_times(0)) or $
      ((window_times(nwindows-1) + halfw*delta_time) gt $
      data_times(ntimes-1)) then $
      message, 'DATA_TIMES do not span WINDOW_TIMES convolved with LEN_WINDOW'
   endelse ; index

xx = reform(x, ntimes, nfuncs)		; Treat all dims after first as one
xw = make_array(npts_window, nwindows, nfuncs, type=type(x))

for windowx=0,nwindows-1 do begin

   if keyword_set(index) then $
      centrex = window_times(windowx) $
   else begin
      w = where(data_times eq window_times(windowx), wcount)
      if wcount eq 0 then $
         message, 'WINDOW_TIMES(' + strim(windowx) + $
            ') not an element of DATA_TIMES' $
      else $
         centrex = w(0)
      endelse

   xw(0,windowx,0) = $
      reform(xx(centrex-halfw:centrex+halfw,*), npts_window, 1, nfuncs)
   endfor

; Restore other dimensions of X

return, reform(xw, [npts_window, nwindows, other_dims])
end
