;+
; NAME:
;	REGION_MULTI
; PURPOSE:
;	Divide an arbitrary rectangle on device into subregions for plotting.
;	Similar to !P.MULTI, but may divide up only part of device area.
;	Calls to the procedure may be nested, so that an arbitrary part of
;	a subregion may be further subdivided.
; CALLING SEQUENCE:
;	To define the main rectangle (containing all subregions) on device,
;	optionally plotting a title:
;	   REGION_MULTI, /INITIALIZE, COLROW [,POSITION=POS] $
;	      [,TITLE=TITLE] [,CHARSIZE=CHARSIZE]
;	To select a subregion:
;	   REGION_MULTI, COLROW [,POSITION=POS] $
;	To reset system variables:
;	   REGION_MULTI, /RESET
;	Note that REGION_MULTI (without /INIT or /RESET) must be called to 
;	select the first and each successive subregion, even though the number
;	of that subregion may be determined automatically by omitting
;	COLROW.
;
;	In nested use, REGION_MULTI, /INIT saves the current subregion
;	description on a stack (including which subregion is selected), and
;	defines a new main rectangle inside and relative to !P.REGION (which 
;	will usually be equal to or within the currently selected subregion).
;	REGION_MULTI, /RESET destroys the current subregion description and
;	restores the previous description from the stack.
;
;	The following is an example of nested calls. Four subregions are 
;	placed side by side on the page, and two plots (one above the other)
;	are put into each subregion.
;
;	   REGION_MULTI, /INIT, [1,4], POS=POS1
;	   FOR I=0,3 DO BEGIN
;	      REGION_MULTI, I, POS=POS2
;
;	      REGION_MULTI, /INIT, [2,1], POS=POS3
;	      FOR J=0,1 DO BEGIN
;	         REGION_MULTI, J, POS=POS4
;	         PLOT, ...
;	         ENDFOR ; J
;	      REGION_MULTI, /RESET
;
;	      ENDFOR ; I
;	   REGION_MULTI, /RESET
;
; INPUTS:
;	COLROW: Integer scalar or vector.
;	      If INITIALIZE is present and non-zero then COLROW must be
;	   present and must be a 2-element vector [NX,NY] where NX =
;	   number of columns and NY = number of rows. 
;	      Otherwise COLROW selects the subregion. Then COLROW may be
;	   either a 2-element vector [index of column of subregion, index
;	   of row of subregion] where indices run from 0 to NX-1 or NY-1,
;	   or COLROW may be a scalar index of the subregion in the range
;	   0 to NX*NY-1 and running from left to right then top to
;	   bottom, or COLROW may be absent in which case the next
;	   subregion is selected in the order in which scalar COLROW
;	   would select. 
; KEYWORD PARAMETERS:
;	POSITION = 4-element real vector.
;	      If INITIALIZE is present and non-zero then POSITION is the
;	   coordinates of the rectangle [x0,y0,x1,y1] which is to
;	   be divided up. POSITION is relative to !P.REGION, or if !P.REGION 
;	   is not set (detected by !P.REGION(0) EQ !P.REGION(2)) then POSITION
;	   is taken to be in normalized coordinates.
;	      Otherwise POSITION is the position of the Plot Region (!P.REGION)
;	   expressed relative to the subregion: [x0,y0,x1,y1] where a value of
;	   0.0 corresponds to the left or bottom edge of the subregion, and
;	   1.0 to the top or right edge. 
;	INITIALIZE: If present and non-zero then all subregions are initialized,
;	   first saving the current subregions description if nesting.
;	TITLE = Scalar string. If INITIALIZE is set then TITLE is plotted above
;	   the rectangle to be divided up. Otherwise ignored.
;	CHARSIZE = Real scalar. Character size for TITLE. Ignored if INITALIZE
;		and TITLE not set.
;	RESET: If present and non-zero then the system variable !P.REGION 
;	   is set to zero, and then the previous subregions 
;	   description is restored if nesting. All other parameters are ignored.
;	CLEAR: If present and non-zero then the system variable !P.REGION 
;	   is set to zero, all other parameters are ignored, and 
;	   internal variables are reset to the state before the first /INIT
;	   call. This is useful for restarting after a program aborts before
;	   all /RESET calls are completed.
; COMMON BLOCKS:
;	CMN_REGMUL
; SIDE EFFECTS:
;	Changes the system variable !P.REGION.
;	After a call without /INIT or /RESET, !P.REGION contains the 
;	normalized coordinates of the selected subregion.
;	Generates graphics output.
; RESTRICTIONS:
;	May be nested a maximum depth of six calls.
; PROCEDURE:
;	Sets !P.REGION to establish a subregion, or optionally a subarea of 
;	the subregion.
;	For nesting, the procedure maintains a stack.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 24 Dec 1992: Written in IDL2, by merging and 
;	   overhauling REGION_MULTI and REGION_POSITION.
;	IFG, 15 Feb 1993: Allowed COLROW to be a one-element array.
;	IFG, 25 Mar 1993: Made nestable.
;	IFG, 13 May 1993: Added CLEAR keyword.
;	GRANT, 27 Sep 1993: Keyword POSITION (without /INIT) no longer sets 
;	   !P.POSITION within the subregion, but now optionally sets !P.REGION
;	   to within the subregion.
;	GRANT, 19 Apr 1994: No longer fails if called with /RESET without a 
;	   previous call with /INIT.
;-

pro region_multi, colrow, position=pos, $
   initialize=init, title=title, charsize=charsize, reset=reset, clear=clear

on_error, 2
debug = !false	;%%

; Save between calls

common cmn_regmul, regmul$nx, regmul$ny, regmul$pos, regmul$colrow, $
   regmul$stack, regmul$stackn, regmul$stackx

; Reset
; =====

if keyword_set(reset) then begin

   if n_elements(regmul$stackx) eq 0 then begin
      message, /inform, 'Never initalized, so no action on /RESET'
   endif else begin

      ; Handle nesting

      if regmul$stackx lt -1 then $
         message, 'Nothing to reset from. Stack empty.'
      if regmul$stackx ge 0 then begin

         ; Pop parameters from stack

         regmul$nx     = regmul$stack(regmul$stackx).nx
         regmul$ny     = regmul$stack(regmul$stackx).ny
         regmul$pos    = regmul$stack(regmul$stackx).pos
         regmul$colrow = regmul$stack(regmul$stackx).colrow
         endif ; regmul$stackx ge 0
      regmul$stackx = regmul$stackx - 1

      ; Finished with nesting

      ; Reset !p.region to default
      !p.region = 0
      if keyword_set(debug) then begin	;%%
         print, 'RESET to: regmul$stackn = ', regmul$stackn, $
            ', regmul$stackx = ', regmul$stackx
         print, '   regmul$nx     = ', regmul$nx
         print, '   regmul$ny     = ', regmul$ny
         print, '   regmul$pos    = ', regmul$pos
         print, '   regmul$colrow = ', regmul$colrow
         endif ; debug
      endelse ; n_elements(regmul$stackx) eq 0

   return
   endif ; reset

; Initialize
; ==========

if keyword_set(init) then begin

   ; Handle nesting

   if n_elements(regmul$stackx) eq 0 then begin
      ; First /INIT
      regmul$stackx = -2
      regmul$stackn = 5		; Maximum stack size
      endif
   if regmul$stackx ge regmul$stackn-1 then $
      message, 'Nesting too deep.'
   regmul$stackx = regmul$stackx + 1
   if regmul$stackx ge 0 then begin

      ; Push previous parameters onto stack

      if n_elements(regmul$stack) eq 0 then $	; Create stack
         regmul$stack = replicate( $
            {regmul$stackstu, nx:0, ny:0, pos:fltarr(4), colrow:0} $
            , regmul$stackn)
      regmul$stack(regmul$stackx).nx     = regmul$nx
      regmul$stack(regmul$stackx).ny     = regmul$ny
      regmul$stack(regmul$stackx).pos    = regmul$pos
      regmul$stack(regmul$stackx).colrow = regmul$colrow
      endif ; regmul$stackx ge 0

   ; Finished with nesting.
   ; Find POS_REGIONS: position in normalized coordinates of all regions.
   ; POS is position of all regions relative to !P.REGION.

   if (not keyword_set(pos)) then pos = [0.0, 0.0, 1.0, 0.9] ; Room for title
   pos_regions = subrect(/to_abs, pregion(), pos)

   ; Plot title

   if n_elements(title) gt 0 then begin
      if n_elements(charsize) eq 0 then charsize = !p.charsize

      ; Setting X and Y is awkward but appears needed for PLOT to work always.

      if !x.range(0) eq 0 and !x.range(1) eq 0 then $
         x = [0,1] $
      else $
         x = !x.range
      if !y.range(0) eq 0 and !y.range(1) eq 0 then $
         y = [0,1] $
      else $
         y = !y.range

      plot, x, y, /nodata, /noerase, position=pos_regions, /norm, $
         xstyle=4, ystyle=4, title=title, $
         charsize=0.8*charsize	; Counteract standard 1.25 factor for title
      endif ; title

   ; Save parameters describing all regions

   regmul$nx     = colrow(0)	; Number of rows
   regmul$ny     = colrow(1)	; Number of columns
   regmul$pos    = pos_regions	; Position of all regions
   regmul$colrow = -1		; Scalar index of previous subregion

   if keyword_set(debug) then begin	;%%
      print, 'INIT  to: regmul$stackn = ', regmul$stackn, $
         ', regmul$stackx = ', regmul$stackx
      print, '   regmul$nx     = ', regmul$nx
      print, '   regmul$ny     = ', regmul$ny
      print, '   regmul$pos    = ', regmul$pos
      print, '   regmul$colrow = ', regmul$colrow
      endif ; debug
   return
   endif ; init

; Clear
; =====

if keyword_set(clear) then begin
   if n_elements(regmul$stackx) eq 0 then begin
      message, /inform, 'Never initalized, so no action on /CLEAR'
   endif else begin
      regmul$stackx = -2
      !p.region = 0	; Reset to default
      endelse
   return
   endif ; clear

; Called without INIT, RESET or CLEAR: Set one region
; ===================================================

; Check for initialization

initialized = !true
if n_elements(regmul$stackx) eq 0 then $
   initialized = !false $
else $
   if regmul$stackx eq -2 then $
      initialized = !false
if not initialized then message, 'Must call with /INIT first'

nx = regmul$nx	; For brevity
ny = regmul$ny
if n_elements(colrow) eq 0 then colrow = regmul$colrow+1
case n_elements(colrow) of
   1:    begin
         ix = (colrow)(0) mod regmul$nx
         iy = (colrow)(0) / regmul$nx
         end
   2:    begin
         ix = colrow(0)
         iy = colrow(1)
         end
   else: message, 'COLROW must have either 1 or 2 elements'
   end
if (ix ge nx) or (iy ge ny) then message, $
   'Subregion requested by COLROW=['+ $
   strconcatenate(strim(colrow),",") + '] is outside NX*NY='+ $
      strim(nx)+'*'+strim(ny)+' limits'
regmul$colrow = iy*regmul$nx + ix
!p.region = subrect(/to_abs, regmul$pos, $
   [float(ix)/nx, float(ny-1-iy)/ny, float(ix+1)/nx, float(ny-iy)/ny])

; Optionally set !P.REGION to a rectangle within the selected subregion.
; POS is position of the rectangle relative to the current subregion.

if keyword_set(pos) then $
   !p.region = subrect(/to_abs, !p.region, pos)

return
end
