;+
; NAME:
;	SPECTRUM
; PURPOSE:
;	Function returns MEM or FFT power spectrum. Frequency in physical units.
; CALLING SEQUENCE:
;	Result = SPECTRUM(X)
; INPUTS:
;	X = Real time series vector(s). If X has more than one dimension, then
;           the first dimension is treated as the time index, while the
;           remaining dimensions index different time series.
; KEYWORD PARAMETERS:
;	DTIME = Sampling period. FLOAT scalar.
;	MEM = scalar. If present and non-zero then MEM (Maximum Entropy Method) 
;		spectra are returned. Default.
;	FFT = scalar. If present and non-zero then magnitude of Fourier 
;		transform is returned.
;
;	MEM and FFT are mutually exclusive.
;
;	ORDER = Order of MEM spectra. Ignored for FFT. Integer scalar.
;	FMAX = Maximum frequency of returned spectra. FLOAT scalar. Default is 
;		Nyquist frequency = half sampling rate.
;	NFSTEPS = Number of frequency steps of returned spectra. FLOAT scalar.
;		The returned spectrum has values for NFSTEPS+1 frequencies.
;		Default is NFSTEPS = 100.
;	F_IN = Vector of frequencies at which the spectrum is to be evaluated.
;		Ignored for FFT.
;	F_OUT = Vector of frequencies at which the spectrum was evaluated.
;		If F_IN was supplied then F_OUT = F_IN.
;
;	MEM can return the spectrum at an arbitrary set of frequencies. If F_IN
;	is not supplied, FMAX and NFSTEPS are used, possibly with defaults.
;	For FFT the frequencies at which the spectrum is calculated are fixed,
;	but this routine allows the return of just the start of the spectrum,
;	either up to FMAX, or the first NFSTEPS+1 frequencies.
;
;	INFORM = scalar. If present and non-zero, progress messages are printed.
;	DETREND: If present and non-zero, a linear trend (least squares fit) 
;		subracted before applying the optional window or calculating
;		the spectrum.
;	HANNING: If present and non-zero, the time series is multiplied by a 
;		Hanning window before the spectrum is calculated.
; OUTPUTS:
;	Result = Spectrum (PSD). Same number of dimensions as X. First 
;	         dimension is the number of frequencies. Other dimensions the
;	         same as X.
; PROCEDURE:
;	Calls CMEM or FFT to find spectrum for normalized frequency.
; Example:
;	Consider a vector X of data sampled at 5 second intervals. The sample 
;	rate is 200 mHz and the Nyquist frequency is 100 mHz.
;	SPECTRUM(x, DTIME=5*1e-3, FMAX=10, F_OUT=f, /MEM) returns the 
;	MEM spectrum (PSD) evaluated from 0 to 10.0 mHz in 0.1 mHz steps. By
;	giving DTIME in kiloseconds, the returned frequencies are in
;	kiloseconds^-1 = millihertz.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 23 Jul 1992: Written in IDL2.
;	IFG 16 Dec 1992: Renamed DELTA_TIME to DTIME. Added ON_ERROR.
;	GRANT, 12 May 1994: Added keywords, DETREND, HANNING.
;-

function spectrum, x, dtime=dtime, $
   mem=mem, fft=fft, order=order, detrend=detrend_on, hanning=hanning_on, $
   fmax=fmax, nfsteps=nfsteps, f_in=f_in, f_out=f_out, inform=inform

on_error, 2
sx = size(x)
ndims = sx(0)
if ndims eq 0 then message, 'X must have at least 1 dimension'
n = sx(1)		; Number of points per function

nfuncs = 1		; Number of functions
if ndims gt 1 then $
   for dx=2,ndims do $
      nfuncs = nfuncs*sx(dx)

w = where([keyword_set(mem), keyword_set(fft)], wcount)
case wcount of
   0: method = 'MEM'
   1: method = (['MEM', 'FFT']) (w(0))
   else: message, 'Must set not more than one of /MEM, /FFT'
   endcase

if not keyword_set(dtime) then dtime = 1.0
fnyquist = 0.5/dtime
if not keyword_set(order) then order = n/4
if not keyword_set(inform) then inform = !false

if method eq 'MEM' then begin

   ; MEM can return PSD at an arbitrary set of frequencies.
   ; If F_IN not supplied, use FMAX and NFSTEPS, possibly with defaults.

   if not keyword_set(fmax) then fmax = fnyquist
   if not keyword_set(nfsteps) then nfsteps = 100
   nf = nfsteps+1
   if keyword_set(f_in) then $
      f_out = f_in $
   else $
      f_out = fmax/float(nfsteps)*findgen(nf)
   nfsteps = n_elements(f_out)-1
endif else begin

   ; For FFT the returned frequencies are fixed, but this routine 
   ; allows the return of just the first NFSTEPS+1 frequencies.

   fstepfull = 2.0*fnyquist/n
   nfstepsfull = n/2			; (n-1)/2 if n odd
   if (keyword_set(fmax) and keyword_set(nfsteps)) then $
      message, 'For FFT, must set only one of FMAX, NFSTEPS'
   if (not keyword_set(fmax)) and (not keyword_set(nfsteps)) then $
      nfsteps = nfstepsfull  ; No freq range chosen, so return all up to Nyquist
   if keyword_set(fmax) then $
      nfsteps = fix(fmax/fstepfull)	; Rounded down if n odd
   nfsteps = min([nfsteps,nfstepsfull])
   f_out = fstepfull*findgen(nfsteps+1)
   endelse

if keyword_set(hanning_on) then $
   window_function = hanning(n)
xx = reform(x, n, nfuncs)		; Treat all dims after first as one
specs = fltarr(n_elements(f_out), nfuncs)
if inform then $
   print, 'Calculating spectra for ' + strim(nfuncs) + ' time series:'
for funcx=0,nfuncs-1 do begin
   if inform then $
      print, funcx, format='($, i5)'
   x1 = xx(*,funcx)
   if keyword_set(detrend_on) then $
      x1 = detrend(x1)
   if keyword_set(hanning_on) then $
      x1 = x1 * window_function
   case method of
      'MEM'   : begin
                spec = cmem(x1, order, f_in=f_out*dtime, f_out=f_out_check)
                spec = 2.0*spec		; 2* for negative freqs
                end
      'FFT'   : begin
                spec = fft(x1, -1)
                spec = spec(0:n/2)	; Discard non-negative frequencies
                spec = n * (float(spec)^2 + imaginary(spec)^2)
                spec(1) = 2.0*spec(1:(n-1)/2)  ; Double all but zero and Nyquist
                end
      endcase
      ;%% help, f_out, fmax, nf, fnyquist, method, nfsteps, nfstepsfull, $
      ;%% fstepfull, f_out_check, spec
      ;%% print, 'ends(f_out) = ', ends(f_out)
   specs(0,funcx) = spec(0:nfsteps)*dtime
   endfor ; funcx
if inform then $
   print, ' '

; Restore to original dimensions

specsdims = [n_elements(f_out)]
if nfuncs gt 1 then $
   specsdims = [specsdims, sx(2:ndims)]
return, reform(specs, specsdims)
end
