;+
; NAME:
;	SUBRECT
; PURPOSE:
;	Convert between absolute and relative coordinates of a subrectangle.
;	Relative coordinates are defined such that the main rectangle extends 
;	from 0 to 1 in both the X and Y directions.
; CALLING SEQUENCE:
;	Result = SUBRECT (MAIN, SUB [,/TO_RELATIVE] [,/TO_ABSOLUTE])
; INPUTS:
;	All rectangles are described by 4-element vectors [x0,y0,x1,y1].
;	MAIN = 4-element vector. Absolute coordinates of main rectangle.
;	   The "relative coordinates" of the main rectangle are [0,0,1,1] by
;	   definition.
;	SUB = 4-element vector. Input coordinates of the subrectangle.
;	   If TO_RELATIVE is set then SUB is interpreted as absolute coords.
;	   If TO_ABSOLUTE is set then SUB is interpreted as relative coords.
; KEYWORD PARAMETERS:
;	TO_RELATIVE: If present and nonzero then the absolute coordinates of 
;	   the subrectangle are converted to relative coordinates.
;	TO_ABSOLUTE: If present and nonzero then the relative coordinates of 
;	   the subrectangle are converted to absolute coordinates.
; OUTPUTS:
;	Result = 4-element vector. Coordinates [x0,y0,x1,y1] of subrectangle.
;	   If TO_RELATIVE is set then Result is relative coordinates.
;	   If TO_ABSOLUTE is set then Result is absolute coordinates.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 4 Jan 1993: Written in IDL2 by merging 
;	   SUBRECT_ABS and SUBRECT_REL.
;-

function subrect, main, sub, to_relative=to_rel, to_absolute=to_abs

on_error, 2
if howmany( [keyword_set(to_rel), keyword_set(to_abs)] ) ne 1 then $
   message, 'Must set exactly one of keywords TO_RELATIVE, TO_ABSOLUTE'

if keyword_set(to_abs) then $
   ; Relative to absolute
   return, [main(0) + sub(0)*(main(2)-main(0)), $	; x0
            main(1) + sub(1)*(main(3)-main(1)), $	; y0
            main(0) + sub(2)*(main(2)-main(0)), $	; x1
            main(1) + sub(3)*(main(3)-main(1))] $	; y1
else $
   ; Absolute to relative
   return, [(sub(0)-main(0)) / (main(2)-main(0)), $	;x0
            (sub(1)-main(1)) / (main(3)-main(1)), $	;y0
            (sub(2)-main(0)) / (main(2)-main(0)), $	;x1
            (sub(3)-main(1)) / (main(3)-main(1))]	;y1

end
