;+
; NAME:
;	STRCANRANGE
; PURPOSE:
;	String date and/or time of start and/or end of vector of CANOPUS times.
; CALLING SEQUENCE:
;	Result = STRCANRANGE (TIMES [, START=start] [, ENDD=endd]
;		                    [, DATE=date]   [, TIME=time] )
; INPUTS:
;	TIMES = Vector of CANOPUS times.
; KEYWORD PARAMETERS:
;	START: If present and non-zero, the result will include the start.
;	ENDD:  If present and non-zero, the result will include the end.
;	If both or neither START and ENDD is set then both start and end will 
;	   be included in the result.
;
;	DATE: If present and non-zero, dates will be included in result.
;	TIME: If present and non-zero, times will be included in result.
;	If both or neither DATE and TIME is set then both dates and times will 
;	   be included in the result.
;
;	NOSECONDS: If present and non-zero, times do not include seconds.
; OUTPUTS:
;	Result = string. Dates as DD-Mth-YYYY, times as HH:MM:SS. Spaces and 
;	   "-" in string depending on combination of start/end dates/times.
;	   Default (no keywords) is start date, start time, end time as
;	   DD-Mth-YYYY HH:MM:SS-HH:MM:SS
; PROCEDURE:
;	Calls STRCANTIME for first and last element of argument vector.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 7 Nov 1991: Written in IDL2 by adapting 
;	   CANTIME2STRINGS.
;       IFG 27 Jul 1992: Added ON_ERROR.
;	IFG, 30 Mar 1993: Added keyword NOSECONDS.
;-

function strcanrange, $
   times, start=start, endd=endd, date=date, time=time, noseconds=nosec

on_error, 2
ks = keyword_set(start)
ke = keyword_set(endd)
if ks and (not ke) then $
   startend = 'S' $		; Start only
else if (not ks) and ke then $
   startend = 'E' $		; End only
else $
   startend = 'B'		; Both

kd = keyword_set(date)
kt = keyword_set(time)
if kd and (not kt) then $
   datetime = 'D' $		; Date only
else if (not kd) and kt then $
   datetime = 'T' $		; Time only
else $
   datetime = 'B'		; Both

len_time = 8
if keyword_set(nosec) then len_time = 5

n = 0
datetime_strs = strarr(2)
if startend ne 'E' then begin			; Need start date and time
   datetime_strs(n) = strcantime(times(0))
   n = n+1
   endif
if startend ne 'S' then begin			; Need end date and time
   datetime_strs(n) = strcantime(times(n_elements(times)-1))
   n = n+1
   endif
date_strs = strmid(datetime_strs, 0, 11)	; Extract dates
time_strs = strmid(datetime_strs, 12, len_time)	; Extract times

if startend ne 'B' then begin
   case datetime of
      'D': return, date_strs(0)
      'T': return, time_strs(0)
      else: return, date_strs(0) + ' ' + time_strs(0)
      endcase
   endif $
else begin	; Both start and end
   case datetime of
      'D': return, date_strs(0) + ' - ' + date_strs(1)
      'T': return, time_strs(0) + '-' + time_strs(1)
      else: begin	; Date and time
         if date_strs(0) eq date_strs(1) then $
            return, date_strs(0) + ' ' + time_strs(0) + '-' + time_strs(1) $
         else $
            return, date_strs(0) + ' ' + time_strs(0) + ' - ' + $
                    date_strs(1) + ' ' + time_strs(1)
         end ; CASE DATETIME else: Date and time
      endcase
   endelse ; startl xor endl

end
